#ifndef LEXER_H__
#define LEXER_H__ 

#include <stdio.h>
#include <stdbool.h>
#include "token.h"
#include <ruby.h>

#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

typedef struct Lexer
{
  FILE *fp;
  yyscan_t scanner;
  bool isClosed;
  VALUE parser;
  char *screen_filename;
  int newline_count;
} *Lexer;

typedef enum
{
  TokenKind_KEYWORD = 1,
  TokenKind_LITERAL = 2,
} TokenKind;

typedef struct TokenInfo
{
  int line;
  TokenKind kind;
  VALUE value;
  int newline_count;
} *TokenInfo;

#define TokenInfo_set(self, l, k, v, nc) \
  do{ \
    (self)->line = (l); \
    (self)->kind = (k); \
    (self)->value = (v); \
    (self)->newline_count = (nc); \
  }while( 0 )

#define Lexer_C_TOKEN_PROC(name, kind, str) TokenID_C_ ## name,
#define Lexer_LUA_TOKEN_PROC(name, kind, str) TokenID_LUA_ ## name,
  
#define TokenID_OFFSET 256
typedef enum {
  TokenID_FIRST = TokenID_OFFSET - 1,
  Lexer_TOKEN_LIST()
  TokenID_LAST
} TokenID;

#define TokenID_SIZE (TokenID_LAST - TokenID_OFFSET)

#undef Lexer_C_TOKEN_PROC
#undef Lexer_LUA_TOKEN_PROC

void Lexer_yy_push_state(int state, yyscan_t scanner);
void Lexer_yy_pop_state(yyscan_t scanner);

// クラス/モジュール定数のキャッシュ
extern VALUE modLuca;
extern VALUE clsLexer;
extern VALUE clsTokenInfo;
extern VALUE errArgumentError;
extern VALUE errLexerError;

#endif
