#ifndef RUBY_UTIL_H__
#define RUBY_UTIL_H__ 

#include <ruby.h>
#include <stdarg.h>

/**
 * フォーマット文字列fmtを用いて, Ruby上の例外errを発生させる
 * 
 * <tt>rb_raise</tt>とは異なり, Cソース上の位置情報を自動的に埋め込む. 
 * <tt>fmt</tt>は文字列リテラルである必要がある. 
 */
#define rb_raise_at(err, fmt, ...) \
  rb_raise(err, "[%s@%s:%u] " fmt, __func__, __FILE__, __LINE__,  ## __VA_ARGS__)

/**
 * C文字列からRubyのSymbolを得る. 
 * 
 * @param s シンボルへ変換するC文字列
 */
#define rb_intern_sym(s) (ID2SYM(rb_intern((s))))

/**
 * vを引数にしてRubyのKernel.pを呼び出す
 * 
 * @param v 表示するRubyオブジェクト
 */
void rb_p(VALUE v);

VALUE rb_call_method(VALUE recv, const char *method, int argc, ...);
VALUE rb_call_method_by_varg(VALUE recv, const char *method, int argc, va_list varg);

#define rb_call_method_by_ary(recv, method, argc, argv) (rb_funcall2((recv), rb_intern((method)), (argc), (argv)))

#endif
