require 'tempfile'
require 'shellwords'
require 'open3'

module Luca
  class Analyzer
    # Cコード部分にエラーが無いかを検査するクラス
    #
    # @author autopp <autopp.inc@gmail.com>
    #
    class CCodeChecker
      include Varspec::BuiltinMatcher
      
      # @return [Compiler] この検査器を生成したコンパイラ
      attr_reader :compiler
      
      def initialize(compiler)
        expect_var[:compiler].of_instance_to_be Compiler
      end
      
      def check_program(program, filename)
        # c_codeを出力
        expect_var[:program].to_be Ast::C::Program
        expect_var[:filename].to_be String
        
        # 一時ファイルにCコードを書き込む
        cfile = Tempfile.open(File.basename(filename))
        CPrinter.new(filename, cfile, 2, compiler.loc_map)
        cfile.close # 一度閉じる
        cfilename = cfile.path
        
        # clangに掛ける為のファイル名を生成
        ofile = Tempfile.open([File.basename(filename), '.o'])
        ofile.close
        ofilename = ofile.path
        
        # clangに掛ける
        cmd_list = ['clang', '-x', 'c', '-c', '-std=c99', '-o', ofilename, cfilename]
        _, err, status = *Open3.capture3(Shellwords.shelljoin(cmd_list))
        
        return if status.success?
        $stderr.puts err
        exit(1)
      end
    end
  end
end
