#!/usr/bin/env ruby

module Luca
  class Analyzer
    # プログラム全体から<tt>_lua</tt>文を抽出し, トップレベルにその情報を付与する
    # また各Luaプログラム中にあるLua関数の情報も集める
    #
    # @author autopp <autopp.inc@gmail.com>
    #
    class ChunkCollector
      include Varspec::BuiltinMatcher
      
      def collect_chunk(program)
        expect_var[:program].to_be Ast::C::Program
        
        @lua_stmt_list = []
        @bquote_list = []
        @function_stack = []
        @block_stack = []
        program.traverse_lua_by(self, :collect_chunk_by, override: true)
        
        program.lua_stmt_list = @lua_stmt_list
        program.bquote_list = @bquote_list
      end
      
      def collect_from_ast(ast)
        expect_var[:ast].to_be Ast
        ast.collect_chunk_by(self)
        nil
      end
      
      def collect_from_list(list)
        expect_var[:list].to_be ArrayOf(Ast)
        list.each do |ast|
          self.collect_from_ast(ast)
        end
      end
      
      def add_lua_stmt(lua_stmt)
        expect_var[:lua_stmt].to_be Ast::C::LuaStmt
        @lua_stmt_list << lua_stmt
      end
      
      def add_bquote(bquote)
        expect_var[:bquote].to_be Or(Ast::Lua::BackQuoteExpr, Ast::Lua::BackQuoteLhs)
        @bquote_list << bquote
      end
      
      def nest_chunk(chunk)
        expect_var[:chunk].to_be Or(Ast::Lua::Program, Ast::Lua::FunctionLiteral)
        @function_stack.push chunk
        @block_stack.push chunk
        yield
        @block_stack.pop
        @function_stack.pop
      end
      
      def current_chunk
        @function_stack.last
      end
      
      def nest_block(block)
        expect_var[:block].to_be Ast::Lua::Block
        @block_stack.push block
        yield
        @block_stack.pop
      end
      
      def current_block
        @block_stack.last
      end
    end
  end
end
