module Luca
  class Analyzer
    # 全てのASTノードに親ノードを付与する為の解析器クラス
    # 
    # ASTノードは<tt>set_parent_by</tt>メソッドを定義する必要がある. 
    #
    # @author autopp <autopp.inc@gmail.com>
    #
    class ParentSetter
      include Varspec::BuiltinMatcher
      
      # プログラム全体に親ノード情報を付与する
      #
      # @param [Ast::C::Program] program 
      #
      # @return [Ast::C::Program] program
      # 
      def set_parent_for(program)
        expect_var[:program].to_be Ast::C::Program
        set_parent_of_ast(program, nil)
        program
      end
      
      def set_parent_of_ast(ast, parent)
        expect_var[:ast].to_be Maybe(Ast)
        ast.parent = parent
        
        ast.set_parent_by(self)
      end
      
      def set_parent_of_list(list, parent)
        expect_var[:list].to_be ArrayOf(Maybe(Ast))
        list.each do |ast|
          self.set_parent_of_ast(ast, parent)
        end
      end
    end
  end
end
