module Luca
  class Analyzer
    class StackAnalyzer
      include Varspec::BuiltinMatcher

      attr_reader :call_count

      def analyze(program)
        expect_var[:program].of_instance_to_be Ast::C::Program
        @expected_result_stack = []
        @safe_var_map = nil
        program.lua_stmt_list.each do |lua_stmt|
          analyze_ast(lua_stmt.lua_program)
        end
      end

      def analyze_ast(ast, n = nil)
        expect_var[:ast].to_be Ast::Lua
        expect_var[:n].to_be Maybe(Integer)

        ret = if n
                expect_result(n) { ast.analyze_stack_by(self) }
              else
                ast.analyze_stack_by(self)
              end
        fail "expect #{ast.class}#analyze_stack_by to return Integer" unless ret.is_a?(Integer)
        fail "expect #{ast.class}#analyze_stack_by to return non negative" unless ret >= 0
        ret
      end

      def analyze_list(list, n = nil)
        expect_var[:list].to_be ArrayOf(Ast::Lua)
        expect_var[:n].to_be Maybe(Integer)

        list.map do |ast|
          analyze_ast(ast, n)
        end
      end

      def expect_result(n)
        @expected_result_stack.push(n)
        ret = yield
        @expected_result_stack.pop
        ret
      end

      def current_expected_result
        @expected_result_stack.last
      end

      def max(*args)
        [0, *args].max
      end

      MULTI_RET = -1

      def multi_ret
        MULTI_RET
      end

      INDEX_MAX = 0
      INDEX_CUR = 1

      def in_function
        prev_map = @safe_var_map
        prev_call_count = @call_count
        @safe_var_map = {}
        @call_count = 0
        ret = yield
        @safe_var_map = prev_map
        @call_count = prev_call_count
        ret
      end

      def inc_call_count
        @call_count += 1
      end

      def let_safe_var(type, n)
        type = type.to_sym
        expect_var[:n].to_be Integer
        if !@safe_var_map.member?(type)
          @safe_var_map[type] = [n, n]
        else
          max_n, cur_n = @safe_var_map[type]
          max_n = max(cur_n + n, max_n)
          @safe_var_map[type][INDEX_MAX] = max_n
          @safe_var_map[type][INDEX_CUR] = cur_n + n
        end

        ret = yield
        @safe_var_map[type][INDEX_CUR] -= n

        ret
      end

      def let_safe_int(n, &block)
        let_safe_var(:int, n, &block)
      end

      def safe_var_size_map
        @safe_var_map.each_with_object({}) do |(k, v), ret|
          ret[k] = v[INDEX_MAX]
        end
      end

      def escape?(id)
        @program.escape?(id)
      end
    end
  end
end
