module Luca
  class Analyzer
    class TypeChecker
      # ffi-clangに与える入力ファイルを生成するためのプリンタクラス
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class ClangInputGenerator < Printer
        include Varspec::BuiltinMatcher
        # @return [Integer] これまで遭遇したバッククォートの数
        attr_reader :back_quote_count

        def initialize(filename, output, width, loc_map)
          super(filename, output, width, loc_map)
          @back_quote_count = 0
          @lua_stmt_count = 0
          @c_local_count = 0
        end

        def gen_input(program)
          expect_var[:program].to_be Ast::C::Program

          # 解析用のラッパー関数のプロトタイプ宣言を出力
          program.lua_stmt_list.each do |lua_stmt|
            lua_stmt.label = gen_lua_stmt_label
          end

          program.bquote_list.each do |bquote|
            bquote.label = gen_bquote_label
          end

          program.clocal_decl_list.each do |c_local|
            c_local.label = gen_c_local_label
          end

          # プログラム本体を出力
          gen_input_of_ast(program)

          program
        end

        def print_ast(ast)
          expect_var[:ast].to_be Ast

          ast.gen_input_by(self)
          self
        end

        alias_method :gen_input_of_ast, :print_ast
        # alias_method :gen_input_of_list, :print_ast_list

        def gen_input_of_list(list, last_sep = false)
          print_ast_list(list, '', last_sep)
        end

        # バッククォート式用のラッパー関数名を生成
        #
        #
        # @return [String] 生成した関数名
        #
        def gen_bquote_label
          ret = "#{TypeChecker.bquote_label_prefix}#{@back_quote_count}"
          @back_quote_count += 1
          ret
        end

        # <tt>_lua</tt>文用のラッパー関数名を生成
        #
        #
        # @return [String] 生成した関数名
        #
        def gen_lua_stmt_label
          ret = "#{TypeChecker.lua_stmt_label_prefix}#{@lua_stmt_count}"
          @lua_stmt_count += 1
          ret
        end

        def gen_c_local_label
          ret = "#{TypeChecker.c_local_label_prefix}#{@c_local_count}"
          @c_local_count += 1
          ret
        end

        def endl
          Printer.endl
        end
      end
    end
  end
end
