module Luca
  class Ast
    class C
      # Cの配列型の宣言子を表すASTノード
      # 
      # 可変長配列宣言子は{VariableArrayDeclarator}を参照
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class ArrayDeclarator < DirectDeclarator
        @is_abstract = false
        
        # @return [DirectDeclarator,nil] 
        attr_reader :base_declarator
        
        # @return [Token]
        attr_reader :lbracket_tk
        
        # @return [Token] 前置static
        attr_reader :pre_static_tk
        
        # @return [[TypeQualifier]] 型修飾子リスト
        attr_reader :qualifier_list
        
        # @return [Token] 後置static
        attr_reader :post_static_tk
        
        # @return [Expr, nil] 省略可能なサイズ指定式
        attr_reader :size_expr
        
        # @return [Token]
        attr_reader :rbracket_tk
        
        def initialize(base_declarator, lbracket_tk, pre_static_tk,
                       qualifier_list, post_static_tk, size_expr, rbracket_tk)
          expect_var[:base_declarator].of_instance_to_be Maybe(DirectDeclarator)
          expect_var[:lbracket_tk].of_instance_to_be Token
          expect_var[:pre_static_tk].of_instance_to_be Maybe(Token)
          expect_var[:qualifier_list].of_instance_to_be ArrayOf(TypeQualifier)
          expect_var[:post_static_tk].of_instance_to_be Maybe(Token)
          expect_var[:size_expr].of_instance_to_be Maybe(Expr)
          expect_var[:rbracket_tk].of_instance_to_be Token
          
          if pre_static_tk && post_static_tk
            fail 'pre static and post static given'
          end
          
          if base_declarator
            super(base_declarator.start_pos, rbracket_tk.line)
          else
            super(lbracket_tk.start_pos, rbracket_tk.line)
          end
        end
        
        # サイズが<tt>static</tt>修飾されているか否かを返す
        #
        #
        # @return [Boolean] 
        # 
        def static?
          pre_static_tk || post_static_tk
        end
        
        def name
          base_declarator ? base_declarator.name : nil
        end
        
        def print_by(cxt)
          cxt << base_declarator if base_declarator
          cxt << lbracket_tk
          cxt << pre_static_tk << ' ' if self.pre_static_tk
          qualifier_list.each do |qualifier|
            cxt << qualifier << ' '
          end
          cxt << post_static_tk << ' ' if self.post_static_tk
          cxt << size_expr if size_expr
          cxt << rbracket_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(base_declarator, self)
          cxt.set_parent_of_list(qualifier_list, self)
          cxt.set_parent_of_ast(size_expr, self) if size_expr
        end
      end
    end
  end
end
