module Luca
  class Ast
    class C
      # Cの<tt>__attribute__</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class Attribute < C
        @is_abstract = false
        
        # @return [Token]
        attr_reader :attribute_tk
        
        # @return [Token]
        attr_reader :outer_lparen_tk
        
        # @return [Token]
        attr_reader :inner_lparen_tk
        
        # @return [[Expr, Token]] 
        attr_reader :arg_com_list
        
        # @return [Token]
        attr_reader :inner_rparen_tk
        
        # @return [Token]
        attr_reader :outer_rparen_tk
        
        def initialize(attribute_tk, outer_lparen_tk, inner_lparen_tk,
                       arg_com_list, inner_rparen_tk, outer_rparen_tk)
          expect_var[:attribute_tk].of_instance_to_be Token
          expect_var[:outer_lparen_tk].of_instance_to_be Token
          expect_var[:inner_lparen_tk].of_instance_to_be Token
          expect_var[:arg_com_list].of_instance_to_be ArrayOf(Or(Expr, Token))
          expect_var[:inner_rparen_tk].of_instance_to_be Token
          expect_var[:outer_rparen_tk].of_instance_to_be Token
          super(attribute_tk.line, outer_rparen_tk.line)
        end
        
        def arg_list
          arg_com_list.select.with_index { |_, i| i.even? }
        end
        
        def com_list
          arg_com_list.select.with_index { |_, i| i.odd? }
        end
        
        def print_by(cxt)
          cxt << attribute_tk << outer_lparen_tk << inner_lparen_tk
          
          arg_com_list.each do |x|
            case x
            when Expr
              cxt << x
            when Token
              cxt << x << ' '
            else
              fail "invalid element #{x}"
            end
          end
          
          cxt << inner_rparen_tk << outer_rparen_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(arg_list, self)
        end
      end
    end
  end
end
