module Luca
  class Ast
    class C
      # Cの関数呼出し式を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class CallExpr < Expr
        @is_abstract = false
        
        # @return [Expr] 呼び出す関数
        attr_reader :func_expr
        alias_method :func, :func_expr
        
        # @return [Token]
        attr_reader :lparen_tk
        
        # @return [[Expr, Token]] 実引数とカンマトークンの配列
        attr_reader :arg_com_list
        
        # @return [Token]
        attr_reader :rparen_tk
        
        def initialize(func_expr, lparen_tk, arg_com_list, rparen_tk)
          expect_var[:func_expr].of_instance_to_be Expr
          expect_var[:lparen_tk].of_instance_to_be Token
          expect_var[:arg_com_list].of_instance_to_be ArrayOf(Or(Expr, Token))
          expect_var[:rparen_tk].of_instance_to_be Token
          super(func_expr.start_pos, rparen_tk.line)
        end
        
        def arg_list
          arg_com_list.select.with_index { |_, i| i.even? }
        end
        
        def com_list
          arg_com_list.select.with_index { |_, i| i.odd? }
        end
        
        def print_by(cxt)
          cxt << func_expr << lparen_tk
          arg_com_list.each do |x|
            case x
            when Expr
              cxt << x
            when Token
              cxt << x << ' '
            else
              fail "invalid element #{x}"
            end
          end
          cxt << rparen_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(func_expr, self)
          cxt.set_parent_of_list(arg_list, self)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@func_expr, :@arg_com_list)
        end
      end
    end
  end
end
