module Luca
  class Ast
    class C
      # Cの<tt>case</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class CaseStmt < Stmt
        @is_abstract = false
        
        # @return [Token]
        attr_reader :case_tk
        
        # @return [Expr] 条件式
        attr_reader :expr
        
        # @return [Token]
        attr_reader :col_tk
        
        # @return [Stmt] <tt>self</tt>が表すcase文に対応する文
        attr_reader :stmt
        
        def initialize(case_tk, expr, col_tk, stmt)
          expect_var[:case_tk].of_instance_to_be Token
          expect_var[:expr].of_instance_to_be Expr
          expect_var[:col_tk].of_instance_to_be Token
          expect_var[:stmt].of_instance_to_be Stmt
          super(case_tk.line, stmt.end_pos)
        end
        
        def print_by(cxt)
          cxt.print_token(case_tk, true)
          cxt << ' ' << expr << col_tk << stmt
        end
        
        def perform_translate_lua_by(recv, method, *args, override:, &block)
          @stmt = stmt.translate_lua_by(recv, method, *args, override: override, &block)
          expect_var[:@stmt].to_be Stmt
          self
        end
        
        def perform_traverse_lua_by(recv, method, *args, override:, &block)
          if override && self.class.instance_methods(false).member?(method)
            self.send(method, recv, *args, &block)
          else
            stmt.traverse_lua_by(recv, method, *args, override: override, &block)
          end
          nil
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(expr, self)
          cxt.set_parent_of_ast(stmt, self)
        end
      end
    end
  end
end
