module Luca
  class Ast
    class C
      # Cの複合リテラルを表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class CompoundLiteral < Expr
        @is_abstract = false
        
        # @return [Token]
        attr_reader :lparen_tk
        
        # @return [TypeName] 
        attr_reader :type_name
        
        # @return [Token]
        attr_reader :rparen_tk
        
        # @return [Token]
        attr_reader :lbrace_tk
        
        # @return [[InitializerElement, Token]] 
        attr_reader :element_com_list
        
        # @return [Token, nil]
        attr_reader :ex_com_tk
        
        # @return [Token]
        attr_reader :rbrace_tk
        
        def initialize(lparen_tk, type_name, rparen_tk, lbrace_tk, 
                       element_com_list, ex_com_tk, rbrace_tk)
          expect_var[:lparen_tk].of_instance_to_be Token
          expect_var[:type_name].of_instance_to_be TypeName
          expect_var[:rparen_tk].of_instance_to_be Token
          expect_var[:lbrace_tk].of_instance_to_be Token
          expect_var[:element_com_list].of_instance_to_be ArrayOf(Or(InitializerElement, Token))
          expect_var[:ex_com_tk].of_instance_to_be Maybe(Token)
          expect_var[:rbrace_tk].of_instance_to_be Token
          super(lparen_tk.line, rbrace_tk.line)
        end
        
        def element_list
          element_com_list.select.with_index { |_, i| i.even? }
        end
        
        def com_list
          element_com_list.select.with_index { |_, i| i.odd? }
        end
        
        def print_by(cxt)
          cxt << lparen_tk << type_name << rparen_tk
          cxt << lbrace_tk
          element_com_list.each do |x|
            case x
            when InitializerElement
              cxt << x
            when Token
              cxt << x << ' '
            else
              fail "invalid element #{x}"
            end
          end
          cxt << ex_com_tk << ' ' if ex_com_tk
          cxt << rbrace_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(type_name, self)
          cxt.set_parent_of_list(element_list, self)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@element_list)
        end
      end
    end
  end
end
