module Luca
  class Ast
    class C
      # Cの複合文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class CompoundStmt < Stmt
        @is_abstract = false
        
        # @return [Token]
        attr_reader :lbrace_tk
        
        # @return [[Stmt | Decl]] ブロック内の文もしくは宣言
        attr_reader :item_list
        
        # @return [Token]
        attr_reader :rbrace_tk
        
        def initialize(lbrace_tk, item_list, rbrace_tk)
          expect_var[:lbrace_tk].of_instance_to_be Token
          expect_var[:item_list].of_instance_to_be ArrayOf(Or(Stmt, Decl))
          expect_var[:rbrace_tk].of_instance_to_be Token
          super(lbrace_tk.line, rbrace_tk.line)
        end
        
        def print_by(cxt)
          cxt << lbrace_tk
          cxt.next_level do
            cxt << item_list
          end
          cxt << rbrace_tk
        end
        
        def perform_translate_lua_by(recv, method, *args, override:, &block)
          @item_list = item_list.map do |item|
            item.translate_lua_by(recv, method, *args, override: override, &block)
          end
          expect_var[:@item_list].to_be ArrayOf(Or(Stmt, Decl))
          
          self
        end
        
        def perform_traverse_lua_by(recv, method, *args, override:, &block)
          item_list.each do |item|
            item.traverse_lua_by(recv, method, *args, override: override, &block)
          end
          nil
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(item_list, self)
        end
      end
    end
  end
end
