module Luca
  class Ast
    class C
      # Cの宣言を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class Decl < C
        @is_abstract = false
        
        # @return [[DeclSpecifier]] 
        attr_reader :decl_specifier_list
        
        # @return [[InitDeclarator, Token]]
        attr_reader :init_declarator_com_list
        
        # @return [[Asm, Attribute]] 
        attr_reader :asm_attribute_list
        
        # @return [Token]
        attr_reader :scol_tk
        
        def initialize(decl_specifier_list, init_declarator_com_list, asm_attribute_list, scol_tk)
          expect_var[:decl_specifier_list].of_instance_to_be ArrayOf(DeclSpecifier) > 0
          expect_var[:init_declarator_com_list].of_instance_to_be ArrayOf(Or(InitDeclarator, Token))
          expect_var[:asm_attribute_list].of_instance_to_be ArrayOf(Or(Asm, Attribute))
          expect_var[:scol_tk].of_instance_to_be Token
          
          start_pos = decl_specifier_list.first.start_pos
          
          super(start_pos, scol_tk.line)
        end
        
        def init_declarator_list
          init_declarator_com_list.select.with_index { |_, i| i.even? }
        end
        
        def print_by(cxt)
          cxt.print_ast_list(decl_specifier_list, ' ')
          
          unless init_declarator_list.empty?
            cxt << ' '
            init_declarator_com_list.each do |x|
              case x
              when InitDeclarator
                cxt << x
              when Token
                cxt << x << ' '
              else
                fail "invalid element #{x}"
              end
            end
          end
          
          unless asm_attribute_list.empty?
            cxt << ' '
            cxt.print_ast_list(asm_attribute_list, ' ')
          end
          cxt << scol_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(decl_specifier_list, self)
          cxt.set_parent_of_list(init_declarator_list, self)
          cxt.set_parent_of_list(asm_attribute_list, self)
        end
      end
    end
  end
end
