module Luca
  class Ast
    class C
      # Cの宣言子を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class Declarator < C
        @is_abstract = false
        
        # @return [[Pointer, DeclaratorAttribute]] 
        attr_reader :pointer_attribute_list
        
        # @return [DirectDeclarator,nil]
        attr_reader :direct_declarator
        
        def initialize(pointer_attribute_list, direct_declarator)
          expect_var[:pointer_attribute_list].of_instance_to_be ArrayOf(Or(Pointer, DeclaratorAttribute))
          expect_var[:direct_declarator].of_instance_to_be Maybe(DirectDeclarator)
          
          start_pos = pointer_attribute_list.empty? ? direct_declarator.start_pos : pointer_attribute_list.first.start_pos
          end_pos = direct_declarator ? direct_declarator.end_pos : pointer_attribute_list.last.end_pos
          super(start_pos, end_pos)
        end
        
        def name
          direct_declarator.name
        end
        
        def print_by(cxt)
          cxt.print_ast_list(pointer_attribute_list, ' ', true)
          cxt << direct_declarator if direct_declarator
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(pointer_attribute_list, self)
          cxt.set_parent_of_ast(direct_declarator, self) if direct_declarator
        end
        
        def gen_env_member_by(cxt, member_index)
          cxt.print_ast_list(pointer_attribute_list, ' ')
          cxt.member_index = member_index
          direct_declarator.gen_env_member_by(cxt) if direct_declarator
        end
      end
    end
  end
end
