module Luca
  class Ast
    class C
      # Cの<tt>do</tt>-<tt>while</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class DoWhileStmt < Stmt
        @is_abstract = false
        
        # @return [Token]
        attr_reader :do_tk
        
        # @return [Stmt] 本体の文
        attr_reader :body_stmt
        
        # @return [Token]
        attr_reader :while_tk
        
        # @return [Token]
        attr_reader :lparen_tk
        
        # @return [Expr] 条件式
        attr_reader :cond_expr
        
        # @return [Token]
        attr_reader :rparen_tk
        
        # @return [Token]
        attr_reader :scol_tk
        
        def initialize(do_tk, body_stmt, while_tk, lparen_tk,
                       cond_expr, rparen_tk, scol_tk)
          expect_var[:do_tk].of_instance_to_be Token
          expect_var[:body_stmt].of_instance_to_be Stmt
          expect_var[:while_tk].of_instance_to_be Token
          expect_var[:lparen_tk].of_instance_to_be Token
          expect_var[:cond_expr].of_instance_to_be Expr
          expect_var[:rparen_tk].of_instance_to_be Token
          expect_var[:scol_tk].of_instance_to_be Token
          
          super(do_tk.line, scol_tk.line)
        end
        
        def print_by(cxt)
          cxt << 'do ' << body_stmt << ' while( ' << cond_expr << ' );'
          
          cxt << do_tk << ' ' << body_stmt << ' ' << while_tk
          cxt << lparen_tk << ' ' << cond_expr << ' ' << rparen_tk << scol_tk
        end
        
        def translate_lua_by(recv, method, *args, block, override:)
          @body_stmt = body_stmt.translate_lua_by(recv, method, *args, block, override: override)
          expect_var[:@body_stmt].to_be Stmt
          self
        end
        
        def perform_traverse_lua_by(recv, method, *args, block, override:)
          body_stmt.traverse_lua_by(recv, method, *args, block, override: override)
          nil
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(body_stmt, self)
          cxt.set_parent_of_ast(cond_expr, self)
        end
      end
    end
  end
end
