module Luca
  class Ast
    class C
      # Cの<tt>enum</tt>定義の個々の列挙子を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class EnumConstant < C
        @is_abstract = false
        
        # @return [Token] メンバ名トークン
        attr_reader :name_tk
        
        # @return [Token, nil] '='トークン
        attr_reader :eq_tk
        
        # @return [Expr, nil] メンバ値
        attr_reader :value
        
        def initialize(name_tk, eq_tk, value)
          expect_var[:name_tk].of_instance_to_be Token
          expect_var[:eq_tk].of_instance_to_be Maybe(Token)
          expect_var[:value].of_instance_to_be Maybe(Expr)
          
          if value          
            super(name_tk.line, value.end_pos)
          else
            super(name_tk.line, name_tk.line)
          end
        end
        
        def name
          name_tk.value
        end
        
        def print_by(cxt)
          cxt << name_tk
          
          return unless value
          cxt << ' ' << eq_tk << ' ' << value
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(value, self) if value
        end
      end
    end
  end
end
