module Luca
  class Ast
    class C
      # Cの<tt>enum</tt>修飾子を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class EnumTypeSpecifier < UserTypeSpecifier
        @is_abstract = false
        
        # @return [Token]
        attr_reader :enum_tk
        
        # @return [Token, nil]
        attr_reader :lbrace_tk
        
        # @return [[EnumConstant, Token]] 定数とカンマが交互に並んだリスト
        attr_reader :constant_com_list
        
        # @return [Token, nil]
        attr_reader :rbrace_tk
        
        def initialize(enum_tk, name_tk, lbrace_tk, constant_com_list, rbrace_tk)
          expect_var[:enum_tk].of_instance_to_be Token
          expect_var[:lbrace_tk].of_instance_to_be Maybe(Token)
          expect_var[:constant_com_list].of_instance_to_be ArrayOf(Or(EnumConstant, Token))
          expect_var[:rbrace_tk].of_instance_to_be Maybe(Token)
          
          if rbrace_tk
            super(enum_tk.line, rbrace_tk.line, name_tk)
          else
            super(enum_tk.line, name_tk.line, name_tk)
          end
        end
        
        def constant_list
          constant_com_list.select.with_index { |_, i| i.even? }
        end
        
        def com_list
          constant_com_list.select.with_index { |_, i| i.odd? }
        end
        
        def print_by(cxt)
          cxt << enum_tk << ' '
          cxt << name_tk << ' ' if name_tk
          
          return if constant_list.empty?
          cxt << lbrace_tk
          cxt.next_level do
            constant_com_list.each do |x|
              case x
              when EnumConstant
                cxt << x
              when Token
                cxt << x << ' '
              else
                fail "invalid element: #{x}"
              end
            end
          end
          cxt << rbrace_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(constant_list, self)
        end
      end
    end
  end
end
