module Luca
  class Ast
    class C
      # Cの<tt>for</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class ForStmt < Stmt
        @is_abstract = false
        
        # @return [Token]
        attr_reader :for_tk
        
        # @return [Token]
        attr_reader :lparen_tk
        
        # @return [Expr, Decl, nil] 初期化式/宣言
        attr_reader :init_part
        
        # @return [Token, nil]
        attr_reader :init_scol_tk
        
        # @return [Expr, nil] 条件式
        attr_reader :cond_expr
        
        # @return [Token]
        attr_reader :cond_scol_tk
        
        # @return [Expr, nil] 更新式
        attr_reader :update_expr
        
        # @return [Token]
        attr_reader :rparen_tk
        
        # @return [Stmt] 本体の文
        attr_reader :body_stmt
        
        def initialize(for_tk, lparen_tk, init_part, init_scol_tk, cond_expr,
                       cond_scol_tk, update_expr, rparen_tk, body_stmt)
          expect_var[:for_tk].of_instance_to_be Token
          expect_var[:lparen_tk].of_instance_to_be Token
          expect_var[:init_part].of_instance_to_be Maybe(Or(Decl, Expr))
          expect_var[:init_scol_tk].of_instance_to_be Maybe(Token)
          expect_var[:cond_expr].of_instance_to_be Maybe(Expr)
          expect_var[:cond_scol_tk].of_instance_to_be Token
          expect_var[:update_expr].of_instance_to_be Maybe(Expr)
          expect_var[:rparen_tk].of_instance_to_be Token
          expect_var[:body_stmt].of_instance_to_be Stmt
          
          super(for_tk.line, body_stmt.end_pos)
        end
        
        def print_by(cxt)
          # 'for( ' までの出力
          cxt << for_tk << lparen_tk << ' '
          # 初期化部分の出力
          case init_part
          when Decl
            cxt << init_part << ' '
          when Expr
            cxt << init_part << init_scol_tk << ' '
          else
            cxt << init_scol_tk << ' '
          end
          # 条件部分の出力
          cxt << cond_expr if cond_expr
          cxt << cond_scol_tk << ' '
          # 更新部分の出力
          cxt << update_expr if update_expr
          # 残りの出力
          cxt << ' ' << rparen_tk << ' ' << body_stmt
        end
        
        def perform_translate_lua_by(recv, method, *args, override:, &block)
          @body = body_stmt.translate_lua_by(recv, method, *args, override: override, &block)
          expect_var[:@body].to_be Stmt
          self
        end
        
        def perform_traverse_lua_by(recv, method, *args, override:, &block)
          body_stmt.traverse_lua_by(recv, method, *args, override: override, &block)
          nil
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(init_part, self) if init_part
          cxt.set_parent_of_ast(cond_expr, self) if cond_expr
          cxt.set_parent_of_ast(update_expr, self) if update_expr
          cxt.set_parent_of_ast(body_stmt, self) if body_stmt
        end
      end
    end
  end
end
