module Luca
  class Ast
    class C
      # Cの関数型宣言子を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class FunctionDeclarator < DirectDeclarator
        @is_abstract = false
        
        # @return [DirectDeclarator, nil] 
        attr_reader :base_declarator
        
        # @return [Token]
        attr_reader :lparen_tk
        
        # @return [[ParamDecl, Token]] 仮引数とカンマトークンが交互に並んだリスト
        attr_reader :param_decl_com_list
        
        # @return [Token, nil] 可変長引数の直前のカンマトークン
        attr_reader :varg_com_tk
        
        # @return [Token, nil] 可変長引数トークン
        attr_reader :varg_tk
        
        # @return [Token]
        attr_reader :rparen_tk
        
        def initialize(base_declarator, lparen_tk, param_decl_com_list,
                       varg_com_tk, varg_tk, rparen_tk)
          expect_var[:base_declarator].of_instance_to_be Maybe(DirectDeclarator)
          expect_var[:lparen_tk].of_instance_to_be Token
          expect_var[:param_decl_com_list].of_instance_to_be ArrayOf(Or(ParamDecl, Token))
          expect_var[:varg_com_tk].of_instance_to_be Maybe(Token)
          expect_var[:varg_tk].of_instance_to_be Maybe(Token)
          expect_var[:rparen_tk].of_instance_to_be Token
          
          if base_declarator
            super(base_declarator.start_pos, rparen_tk.line)
          else
            super(lparen_tk.line, rparen_tk.line)
          end
        end
        
        def param_decl_list
          param_decl_com_list.select.with_index { |_, i| i.even? }
        end
        
        def com_list
          param_decl_com_list.select.with_index { |_, i| i.odd? }
        end
        
        def name
          base_declarator ? base_declarator.name : nil
        end
        
        # rubocop:disable Style/TrivialAccessors
        # rubocop:disable Style/PredicateName
        
        # 可変長引数を持っているか否かを返す
        #
        #
        # @return [Boolean] 
        # 
        def has_varg?
          varg_tk != nil
        end
        # rubocop:enable Style/TrivialAccessors
        # rubocop:enable Style/PredicateName 
        
        def print_by(cxt)
          cxt << base_declarator if base_declarator
          
          cxt << lparen_tk
          cxt.next_level do
            param_decl_com_list.each do |x|
              case x
              when ParamDecl
                cxt << x
              when Token
                cxt << x << ' '
              else
                fail "invalid element #{x}"
              end
            end
            
            if self.has_varg?
              cxt << varg_com_tk << ' ' if varg_com_tk
              cxt << varg_tk
            end
          end
          cxt << rparen_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(base_declarator, self) if base_declarator
          cxt.set_parent_of_list(param_decl_list, self)
        end
      end
    end
  end
end
