module Luca
  class Ast
    class C
      # Cの関数定義を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class FunctionDef < C
        @is_abstract = false
        
        # @return [[DeclSpecifier]] 
        attr_reader :decl_specifier_list
        
        # @return [Declarator] 
        attr_reader :declarator
        
        # @return [[Decl]] 
        attr_reader :decl_list
        
        # @return [CompoundStmt] 
        attr_reader :body
        
        # @return [[LuaStmt]] 関数本体に含まれる<tt>_lua</tt>文のリスト
        attr_reader :lua_stmt_list
        
        # @return [[Lua::BackQuoteExpr]] 関数本体に表れる逆クォート式のリスト
        attr_reader :bquote_list
        
        def initialize(decl_specifier_list, declarator, decl_list, body)
          @decl_specifier_list = expect_var[:decl_specifier_list].to_be ArrayOf(DeclSpecifier)
          @declarator = expect_var[:declarator].to_be Declarator
          @decl_list = expect_var[:decl_list].to_be ArrayOf(Decl)
          @body = expect_var[:body].to_be CompoundStmt
          @lua_stmt_list = []
          @bquote_list = []
          super(decl_specifier_list.first.start_pos, body.end_pos)
        end
        
        def print_by(cxt)
          cxt.print_ast_list(decl_specifier_list, ' ')
          cxt << ' ' << declarator
          decl_list.each do |decl|
            cxt << decl
          end
          
          cxt << body
        end
        
        def perform_translate_lua_by(recv, method, *args, override:, &block)
          @body = body.translate_lua_by(recv, method, *args, override: override, &block)
          expect_var[:@body].to_be CompoundStmt
          self
        end
        
        def perform_traverse_lua_by(recv, method, *args, override:, &block)
          body.traverse_lua_by(recv, method, *args, override: override, &block)
          nil
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(decl_specifier_list, self)
          cxt.set_parent_of_ast(declarator, self)
          cxt.set_parent_of_list(decl_list, self)
          cxt.set_parent_of_ast(body, self)
        end
        
        def gen_code_by(cxt)
          current_loc = cxt.current_logical_loc
          current_line = cxt.current_line
          cxt.in_nolmacro_mode do
            # 本体にある_lua文内のclocal環境を表す構造体を生成
            cxt.newline
            cxt.add_comment('Environment for clocal variable')
            lua_stmt_list.each do |lua_stmt|
              program = lua_stmt.program
              program.gen_c_env_by(cxt)
              cxt.newline
              program.inner_function_list.each do |function|
                function.gen_c_env_by(cxt)
                cxt.newline
              end
            end
            
            # 本体にある関数ポインタを返す逆クォート式のラッパー関数を生成
            cxt.add_comment('Wrapper function of back quote expr')
            bquote_list.each do |bquote|
              if bquote.type.function_pointer? && !bquote.at_call_stmt
                bquote.gen_wrapper_function_by(cxt)
                cxt.newline
              end
            end
            
            # 本体にある<tt>_lua</tt>文内の関数のC定義を生成
            lua_stmt_list.each do |lua_stmt|
              lua_stmt.lua_program.inner_function_list.each do |function|
                function.gen_definition(cxt)
                cxt.newline
              end
            end
            cxt.newline
          end
          cxt.write_lmacro(current_line, current_loc.line, current_loc.filename)
          
          print_by(cxt)
        end
      end
    end
  end
end
