module Luca
  class Ast
    class C
      # Cの<tt>if</tt>文を表すAST
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class IfStmt < Stmt
        @is_abstract = true

        # @return [Token]
        attr_reader :if_tk

        # @return [Token]
        attr_reader :lparen_tk

        # @return [Expr] 条件式
        attr_reader :cond_expr

        # @return [Token]
        attr_reader :rparen_tk

        # @return [Stmt] then文
        attr_reader :then_stmt

        # @return [Token,nil]
        attr_reader :else_tk

        # @return [Stmt] else文
        # @return [nil] else文が無い場合
        attr_reader :else_stmt

        def initialize(if_tk, lparen_tk, cond_expr, rparen_tk, then_stmt, else_tk = nil, else_stmt = nil)
          expect_var[:if_tk].of_instance_to_be Token
          expect_var[:lparen_tk].of_instance_to_be Token
          expect_var[:cond_expr].of_instance_to_be Expr
          expect_var[:rparen_tk].of_instance_to_be Token
          expect_var[:then_stmt].of_instance_to_be Stmt
          expect_var[:else_tk].of_instance_to_be Maybe(Token)
          expect_var[:else_stmt].of_instance_to_be Maybe(Stmt)

          if else_stmt
            super(if_tk.line, else_stmt.end_pos)
          else
            super(if_tk.line, then_stmt.end_pos)
          end
        end

        def print_by(cxt)
          cxt << if_tk << lparen_tk << ' ' << cond_expr << ' ' << rparen_tk << ' ' << then_stmt
          return unless else_stmt
          cxt << else_tk << ' ' << else_stmt
        end

        def perform_translate_lua_by(recv, method, *args, override:, &block)
          @then_stmt = then_stmt.translate_lua_by(recv, method, *args, override: override, &block)
          expect_var[:@then_stmt].to_be Stmt

          if else_stmt
            @else_stmt = else_stmt.translate_lua_by(recv, method, *args, override: override, &block)
            expect_var[:@else_stmt].to_be Stmt
          end

          self
        end

        def perform_traverse_lua_by(recv, method, *args, override:, &block)
          then_stmt.traverse_lua_by(recv, method, *args, override: override, &block)
          else_stmt.traverse_lua_by(recv, method, *args, override: override, &block) if else_stmt
          nil
        end

        def set_parent_by(cxt)
          cxt.set_parent_of_ast(cond_expr, self)
          cxt.set_parent_of_ast(then_stmt, self)
          cxt.set_parent_of_ast(else_stmt, self) if else_stmt
        end
      end
    end
  end
end
