module Luca
  class Ast
    class C
      # Cの複合初期化子の個々の要素を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class InitializerElement < C
        @is_abstract = false
        
        # @return [[Designator]] 
        attr_reader :designator_list
        
        # @return [Token, nil] deginatorと初期化子の間の'='トークン
        attr_reader :eq_tk
        
        # @return [Initializer] 
        attr_reader :initializer
        
        def initialize(designator_list, eq_tk, initializer)
          expect_var[:designator_list].of_instance_to_be ArrayOf(Designator)
          expect_var[:eq_tk].of_instance_to_be Maybe(Token)
          expect_var[:initializer].of_instance_to_be Initializer
          start_pos = designator_list.size > 0 ? designator_list.first.start_pos : initializer.start_pos
          super(start_pos, initializer.end_pos)
        end
        
        def print_by(cxt)
          unless designator_list.empty?
            cxt << designator_list
            cxt << ' ' << eq_tk << ' '
          end
          
          cxt << initializer
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(designator_list, self)
          cxt.set_parent_of_ast(initializer, self)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@initializer)
        end
      end
    end
  end
end
