module Luca
  class Ast
    class C
      # Cのラベル付き文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class LabeledStmt < Stmt
        @is_abstract = false
        
        # @return [Token] ラベル部分のトークン
        attr_reader :label_tk
        
        # @return [Token] コロン部分のトークン
        attr_reader :col_tk
        
        # @return [Stmt] ラベル付けされた文
        attr_reader :stmt
        
        def initialize(label_tk, col_tk, stmt)
          expect_var[:label_tk].of_instance_to_be Token
          expect_var[:col_tk].of_instance_to_be Token
          expect_var[:stmt].of_instance_to_be Stmt
          super(start_pos, stmt.end_pos)
        end
        
        def label
          label_token.value
        end
        
        def print_by(cxt)
          cxt.print_token(label_tk, true)
          cxt << col_tk << stmt
        end
        
        def perform_translate_lua_by(recv, method, *args, override:, &block)
          @stmt = stmt.translate_lua_by(recv, method, *args, override: override, &block)
          expect_var[:@stmt].to_be Stmt
          self
        end
        
        def perform_traverse_lua_by(recv, method, *args, override:, &block)
          stmt.traverse_lua_by(recv, method, *args, override: override, &block)
          nil
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(stmt, self)
        end
      end
    end
  end
end
