module Luca
  class Ast
    class C
      # Cのメンバアクセス式を表すASTノード
      # 
      # @author autopp <autopp.inc@gmail.com>
      #
      class MemberAccessExpr < Expr
        @is_abstract = false
        
        # @return [Expr] アクセス対象の式
        attr_reader :object_expr
        
        alias_method :object, :object_expr
        
        # @return [Token]
        attr_reader :op_tk
        
        # @return [Token] アクセスするメンバ名のトークン
        attr_reader :member_name_tk
        
        alias_method :member_tk, :member_name_tk
        alias_method :name_tk, :member_name_tk
        
        def initialize(object_expr, op_tk, member_name_tk)
          expect_var[:object_expr].of_instance_to_be Expr
          expect_var[:op_tk].of_instance_to_be Token
          expect_var[:member_name_tk].of_instance_to_be Token
          super(object_expr.start_pos, member_name_tk.line)
        end
        
        def op
          op_tk.value
        end
        
        def member_name
          member_name_tk.value
        end
        alias_method :member, :member_name
        alias_method :name, :member_name
        
        def print_by(cxt)
          cxt << object_expr << op_tk << member_name_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(object_expr, self)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@object_expr)
        end
      end
    end
  end
end
