module Luca
  class Ast
    class C
      # Cのメンバ宣言を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class MemberDecl < C
        @is_abstract = false
        
        # @return [[TypeSpecfier, TypeQualifier, ExtensionSpecifier]] 宣言するメンバの指定子/修飾子のリスト
        attr_reader :specifier_qualifier_list
        
        alias_method :spec_qual_list, :specifier_qualifier_list
        
        # @return [[MemberDeclarator, Token]] 宣言子とカンマトークンが交互に並んだリスト
        attr_reader :declarator_com_list
        
        # @return [Token]
        attr_reader :scol_tk
        
        def initialize(specifier_qualifier_list, declarator_com_list, scol_tk)
          expect_var[:specifier_qualifier_list].of_instance_to_be ArrayOf(Or(TypeSpecifier, TypeQualifier, ExtensionSpecifier)) > 0
          expect_var[:declarator_com_list].of_instance_to_be ArrayOf(Or(MemberDeclarator, Token))
          expect_var[:scol_tk].of_instance_to_be Token
          
          super(specifier_qualifier_list[0].start_pos, scol_tk.line)
        end
        
        def declarator_list
          declarator_com_list.select.with_index { |_, i| i.even? }
        end
        
        def com_list
          declarator_com_list.select.with_index { |_, i| i.odd? }
        end
        
        def print_by(cxt)
          specifier_qualifier_list.each do |sq|
            cxt << sq << ' '
          end
          
          declarator_com_list.each do |x|
            case x
            when MemberDeclarator
              cxt << x
            when Token
              cxt << x << ' '
            end
          end
          
          cxt << scol_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(specifier_qualifier_list, self)
          cxt.set_parent_of_list(declarator_list, self)
        end
      end
    end
  end
end
