module Luca
  class Ast
    class C
      # Cのメンバ宣言子を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class MemberDeclarator < C
        @is_abstract = false
        
        # @return [Declarator, nil] 宣言子
        attr_reader :declarator
        
        # @return [Token, nil] ビットフィールドの手前のコロンを表すトークン
        attr_reader :col_tk
        
        # @return [Expr, nil] ビットフィールド
        attr_reader :bit_field
        
        def initialize(declarator, col_tk, bit_field)
          if declarator.nil? && bit_field.nil?
            fail 'both declarator and bit_field is not given'
          end
          
          expect_var[:declarator].of_instance_to_be Maybe(Declarator)
          expect_var[:col_tk].of_instance_to_be Maybe(Token)
          expect_var[:bit_field].of_instance_to_be Maybe(Expr)
          
          start_pos = declarator ? declarator.start_pos : col_tk.line
          end_pos = bit_field ? bit_field.end_pos : declarator.end_pos
          super(start_pos, end_pos)
        end
        
        def print_by(cxt)
          cxt.print_ast(declarator) if declarator
          
          return unless bit_field
          cxt << ': '
          cxt.print_ast(bit_field)
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(declarator, self) if declarator
          cxt.set_parent_of_ast(bit_field, self) if bit_field
        end
      end
    end
  end
end
