module Luca
  class Ast
    class C
      # メンバが存在する型修飾子を表すASTノード
      # 
      # @note このクラスは抽象クラスである
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class MemberedTypeSpecifier < UserTypeSpecifier
        @is_abstract = false
        
        # @return [Token] この修飾子が宣言している型の種類(<tt>struct</tt> or <tt>union</tt>)
        attr_reader :kind_tk
        
        # @return [Token, nil]
        attr_reader :lbrace_tk
        
        # @return [[MemberDecl]] メンバ宣言のリスト
        attr_reader :member_decl_list
        
        # @return [Token, nil]
        attr_reader :rbrace_tk
                
        def initialize(kind_tk, name_tk, lbrace_tk = nil, member_decl_list = [], rbrace_tk = nil)
          expect_var[:kind_tk].of_instance_to_be Token
          expect_var[:lbrace_tk].of_instance_to_be Maybe(Token)
          expect_var[:member_decl_list].of_instance_to_be ArrayOf(MemberDecl)
          expect_var[:rbrace_tk].of_instance_to_be Maybe(Token)
          
          if member_decl_list.empty?
            super(kind_tk.line, name_tk.line, name_tk)
          else
            super(kind_tk.line, rbrace_tk.line, name_tk)
          end
        end
        
        def kind
          kind_tk.value
        end
        
        def print_by(cxt)
          cxt << kind_tk << ' '
          cxt << name_tk if name_tk
          
          return if member_decl_list.empty?
          cxt << lbrace_tk          
          cxt.next_level do
            cxt << member_decl_list
          end
          cxt << rbrace_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(member_decl_list, self)
        end
      end
    end
  end
end
