module Luca
  class Ast
    class C
      # Cのプログラム全体を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class Program < C
        @is_abstract = false
        @expected_parent_class = NilClass

        # @return [String] プログラムのファイル名
        attr_reader :filename

        # @return [[Decl, FunctionDef]]
        attr_reader :external_decl_list

        # @return [Set]
        attr_accessor :escape_var_set

        def initialize(filename, external_decl_list = [])
          expect_var[:filename].of_instance_to_be String

          @external_decl_list = expect_var[:external_decl_list].to_be ArrayOf(Or(Decl, FunctionDef))

          if external_decl_list.empty?
            super(1, 1)
          else
            super(external_decl_list.first.start_pos, external_decl_list.last.end_pos)
          end
        end

        def print_by(cxt)
          cxt << external_decl_list << cxt.endl
        end

        def perform_translate_lua_by(recv, method, *args, override:, &block)
          @external_decl_list = external_decl_list.map do |decl|
            decl.translate_lua_by(recv, method, *args, override: override, &block)
          end
          expect_var[:@external_decl_list].to_be ArrayOf(Or(Decl, FunctionDef))
          self
        end

        def perform_traverse_lua_by(recv, method, *args, override:, &block)
          external_decl_list.each do |decl|
            decl.traverse_lua_by(recv, method, *args, override: override, &block)
          end
          nil
        end

        def set_parent_by(cxt)
          cxt.set_parent_of_list(external_decl_list, self)
        end

        def function_def_list
          external_decl_list.select do |decl|
            decl.is_a?(FunctionDef)
          end
        end

        def clocal_decl_list
          ret2 = lua_stmt_list.each_with_object([]) do |lua_stmt, ret|
            program = lua_stmt.lua_program
            ret.concat(program.clocal_decl_list)
            program.inner_function_list.each do |func|
              ret.concat(func.clocal_decl_list)
            end
          end

          expect_var[:ret2].to_be ArrayOf(Ast::Lua::CLocalDeclStmt)
          ret2
        end

        # プログラム中の<tt>_lua</tt>文のリスト(複製)を取得する
        #
        # 未設定の場合は例外を発生させる
        #
        # @return [[LuaStmt]]
        #
        def lua_stmt_list
          unless instance_variable_defined?(:@lua_stmt_list)
            fail 'no @lua_stmt_list setting'
          end

          @lua_stmt_list.dup
        end

        # プログラム中の<tt>_lua</tt>文のリスト複製を設定する
        #
        # @param [[LuaStmt]] list
        #
        # @return [[LuaStmt]] listを返す
        #
        def lua_stmt_list=(list)
          if instance_variable_defined?(:@lua_stmt_list)
            fail 'multiple @lua_stmt_list setting'
          end
          expect_var[:list].to_be ArrayOf(LuaStmt)
          @lua_stmt_list = list
        end

        # プログラム中のバッククォート式のリスト(複製)を取得する
        #
        # 未設定の場合は例外を発生させる
        #
        # @return [[BackQuoteExpr]]
        #
        def bquote_list
          fail 'no @bquote_list setting' unless instance_variable_defined?(:@bquote_list)

          @bquote_list.dup
        end

        def bquote_list=(list)
          fail 'multiple @bquote_list setting' if instance_variable_defined?(:@bquote_list)
          expect_var[:list].to_be ArrayOf(Or(Lua::BackQuoteExpr, Lua::BackQuoteLhs))
          @bquote_list = list
        end

        def escape?(id)
          @escape_var_set.include?(id)
        end
      end
    end
  end
end
