module Luca
  class Ast
    class C
      # Cの<tt>switch</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class SwitchStmt < Stmt
        @is_abstract = false
        
        # @return [Token]
        attr_reader :switch_tk
        
        # @return [Token]
        attr_reader :lparen_tk
        
        # @return [Expr] 条件式
        attr_reader :cond_expr
        
        # @return [Token]
        attr_reader :rparen_tk
        
        # @return [Stmt] 本体の文
        attr_reader :body_stmt
        
        def initialize(switch_tk, lparen_tk, cond_expr, rparen_tk, body_stmt)
          expect_var[:switch_tk].of_instance_to_be Token
          expect_var[:lparen_tk].of_instance_to_be Token
          expect_var[:cond_expr].of_instance_to_be Expr
          expect_var[:rparen_tk].of_instance_to_be Token
          expect_var[:body_stmt].of_instance_to_be Stmt
          super(switch_tk.line, body_stmt.end_pos)
        end
        
        def print_by(cxt)
          cxt << switch_tk << lparen_tk << ' ' << cond_expr << ' ' << rparen_tk
          cxt << ' ' << body_stmt
        end
        
        def perform_translate_lua_by(recv, method, *args, override:, &block)
          @body_stmt = body_stmt.translate_lua_by(recv, method, *args, override: override, &block)
          expect_var[:@body_stmt].to_be Stmt
          self
        end
        
        def perform_traverse_lua_by(recv, method, *args, override:, &block)
          body_stmt.traverse_lua_by(recv, method, *args, override: override, &block)
          nil
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(cond_expr, self)
          cxt.set_parent_of_ast(body_stmt, self)
        end
      end
    end
  end
end
