module Luca
  class Ast
    class C
      # Cの可変長配列宣言子を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class VariableArrayDeclarator < DirectDeclarator
        @is_abstract = false
        
        # @return [DirectDeclarator, nil] 
        attr_reader :base_declarator
        
        # @return [Token]
        attr_reader :lbracket_tk
        
        # @return [[TypeQualifier]] 
        attr_reader :qualifier_list
        
        # @return [Token]
        attr_reader :ast_tk
        
        # @return [Token]
        attr_reader :rbracket_tk
        
        def initialize(base_declarator, lbracket_tk, qualifier_list, ast_tk, rbracket_tk)
          expect_var[:base_declarator].of_instance_to_be Maybe(DirectDeclarator)
          expect_var[:lbracket_tk].of_instance_to_be Token
          expect_var[:qualifier_list].of_instance_to_be ArrayOf(TypeQualifier)
          expect_var[:ast_tk].of_instance_to_be Token
          expect_var[:rbracket_tk].of_instance_to_be Token
          
          if base_declarator
            super(base_declarator.start_pos, rbracket_tk.line)
          else
            super(lbracket_tk.line, rbracket_tk.line)
          end
        end
        
        def name
          base_declarator ? base_declarator.name : nil
        end
        
        def print_by(cxt)
          cxt << base_declarator if base_declarator
          cxt << lbracket_tk
          qualifier_list.each do |qualifier|
            cxt << qualifier << ' '
          end
          cxt << ast_tk << rbracket_tk
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(base_declarator, self) if base_declarator
          cxt.set_parent_of_list(qualifier_list, self) if qualifier_list
        end
      end
    end
  end
end
