module Luca
  class Ast
    class Lua
      # Luaのフィールドアクセスを表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class AccessExpr < Expr
        @is_abstract = false
        
        # @return [Expr] アクセス先の式
        attr_reader :table_expr
        
        # @return [Expr] アクセスするキーの式
        attr_reader :key_expr
        alias_method :key, :key_expr
        
        def initialize(end_pos, table_expr, key_expr)
          expect_var[:table_expr, :key_expr].of_instance_to_be Expr
          
          super(table_expr.start_pos, end_pos)
        end
        
        # ドット演算子によるアクセス式のASTノードを生成する
        #
        # @param [Expr] table_expr アクセス先の式
        # @param [Symbol] name アクセスするキー
        # @param [Integer] name_pos キーの位置
        #
        # @return [AccessExpr] 
        # 
        def self.create_by_dot_access(table_expr, name, name_pos)
          self.new(name_pos, table_expr, StringLiteral.new(name_pos, name.to_s))
        end
        
        def to_lhs
          AccessLhs.new(end_pos, table_expr, key_expr)
        end
        
        def print_by(cxt)
          cxt << table_expr << '[' << key_expr << ']'
        end
        
        def desugar_by(cxt)
          desugar_ivar_by(cxt, :@table_expr, :@key_expr)
          self
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(table_expr, self)
          cxt.set_parent_of_ast(key_expr, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_ast(table_expr)
          cxt.collect_from_ast(key_expr)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@table_expr, :@key_expr)
          self
        end
        
        def analyze_stack_by(cxt)
          table_s, key_s = cxt.expect_result(1) do
            [cxt.analyze_ast(table_expr), cxt.analyze_ast(key_expr)]
          end
          
          [table_s, key_s + 1].max
        end
        
        def gen_input_by(cxt)
          cxt << table_expr << key_expr
        end
        
        def gen_code_by(cxt)
          # テーブルをスタックに積む
          cxt << table_expr << cxt.endl
          
          # キーをスタックに積む
          cxt << key_expr << cxt.endl
          
          # 値を取得
          cxt.call_api('gettable', -2)
          
          # テーブルを取り除く
          cxt.call_api('remove', -2)
        end
      end
    end
  end
end
