module Luca
  class Ast
    class Lua
      # Luaのフィールドアクセスの左辺値を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class AccessLhs < Lhs
        @is_abstract = false

        # @return [Expr] アクセス先の式
        attr_reader :table_expr
        alias_method :table, :table_expr

        # @return [Expr] アクセスするキーの式
        attr_reader :key_expr
        alias_method :key, :key_expr

        def initialize(end_pos, table_expr, key_expr)
          expect_var[:table_expr, :key_expr].of_instance_to_be Expr
          super(table_expr.start_pos, end_pos)
        end

        def print_by(cxt)
          cxt << table_expr << '[' << key_expr << ']'
        end

        def desugar_by(cxt)
          desugar_ivar_by(cxt, :@table_expr, :@key_expr)
          self
        end

        def set_parent_by(cxt)
          cxt.set_parent_of_ast(table_expr, self)
          cxt.set_parent_of_ast(key_expr, self)
        end

        def collect_chunk_by(cxt)
          cxt.collect_from_ast(table_expr)
          cxt.collect_from_ast(key_expr)
        end

        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@table_expr, :@key_expr)
        end

        def analyze_stack_of_lvalue_by(cxt)
          table_usage = cxt.analyze_ast(table_expr, 1)
          key_usage = cxt.analyze_ast(key_expr, 1) + 1
          cxt.max(table_usage, key_usage)
        end

        def analyze_stack_of_assignment_by(cxt)
          # キーをpushvalueする分
          1
        end

        def assignment_usage(cxt)
          1
        end

        def gen_input_by(cxt)
          cxt << table_expr << key_expr
        end

        def stack_usage(cxt)
          2
        end

        # 左辺値の計算を行うコードを生成する．
        #
        # テーブルとキーを積む
        #
        # @param [CodeGenerator] cxt
        #
        # @return [nil]
        #
        def gen_lvalue_code_by(cxt)
          cxt << table_expr << cxt.endl << key_expr << cxt.endl
          nil
        end

        def gen_assignment_code_by(cxt, state, index)
          # キーをコピーして2番目に挿入
          cxt.call_api('pushvalue', index + 1)
          cxt.call_api('insert', -2)

          # 代入
          cxt.call_api('settable', index - 1)
        end
      end
    end
  end
end
