module Luca
  class Ast
    class Lua
      class BackQuoteCallStmt < Stmt
        # @return [FunctionCallExpr]
        attr_reader :body
        def initialize(body)
          expect_var[:body].of_instance_to_be FunctionCallExpr
          body.func_expr.at_call_stmt = true
          super(body.start_pos, body.end_pos)
        end

        def set_parent_by(cxt)
          cxt.set_parent_of_ast(body, self)
        end

        def collect_chunk_by(cxt)
          cxt.collect_from_ast(body)
        end

        def analyze_jump_by(cxt); end

        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@body)
        end

        def analyze_stack_by(cxt)
          func_expr, arg_list = body.func_expr, body.arg_list
          arg_usage_list = arg_list[0...-1].map.with_index do |arg, i|
            cxt.analyze_ast(arg, 1) + i + 1
          end

          arg_usage = arg_usage_list.max || 0
          last_usage = if arg_list.empty?
                         0
                       else
                         param_size = func_expr.type.pointee.num_arg_types
                         arg_size = arg_list.size
                         cxt.analyze_ast(arg_list.last, param_size - arg_size + 1) + arg_size - 1
                       end
          1 + cxt.max(arg_usage, last_usage)
        end

        def gen_input_by(cxt)
          cxt.gen_input_of_ast(body)
        end

        def gen_code_by(cxt)
          func_expr, arg_list = body.func_expr, body.arg_list
          type = func_expr.type
          func_type = type.pointee
          param_size = func_type.num_arg_types

          if func_expr.at_call_stmt
            if !type.function_pointer?
              cxt.compiler.add_error(func_expr.start_pos, "expect function but got #{type.spelling}")
            else
              # 実引数の個数が一致しているか否かをチェック
              arg_size = body.arg_list.size
              param_size = type.pointee.num_arg_types
              if !body.multi_last_arg? && arg_size != param_size && !type.pointee.variadic?
                cxt.compiler.add_error(func_expr.start_pos, "expected #{param_size} arguments, but got #{arg_size}")
              elsif arg_size > param_size && !type.pointee.variadic?
                cxt.compiler.add_error(func_expr.start_pos, "expected #{param_size} arguments, but got to many arguments")
              end
            end
          end

          # 関数を評価してスタックにライトユーザデータとして積む
          cxt.newline
          func_var = cxt.gen_name(:func)
          cxt.add_stmt(type.gen_decl(func_var))
          cxt << "#{func_var} = "
          cxt.in_lmacro_mode do
            cxt << func_expr.expr
          end
          cxt << ';' << cxt.endl
          cxt.call_api('pushlightuserdata', func_var)

          # 引数を評価してスタックに積む
          arg_list.each do |arg|
            cxt << arg
          end

          # 引数を変換して呼び出す
          arg_var_list = param_size.times.map do |i|
            cxt.gen_name("arg#{i + 1}")
          end

          func_type.each_arg_type.with_index do |t, i|
            arg_var = arg_var_list[i]
            stack_idx = -(param_size - i)
            cxt.gen_checking_type_code(stack_idx, t)
            decl = t.gen_decl(arg_var)

            api = if t.integer?
                    cxt.build_api('tointeger', stack_idx)
                  elsif t.float_point?
                    cxt.build_api('tonumber', stack_idx)
                  elsif t.string?
                    cxt.build_api('tostring', stack_idx)
                  else
                    "*(#{t.gen_decl('*')})#{cxt.build_api('touserdata', stack_idx)}"
                  end
            cxt.add_stmt("#{decl} = #{api}")
          end

          # スタックに保存しておいた関数を呼び出す
          cxt.add_stmt("#{func_var} = " + cxt.build_api('touserdata', -(param_size + 1)))
          cxt.add_stmt("#{func_var}(#{arg_var_list.join(',')})")

          # 関数と引数をスタックから除去
          cxt.call_api('pop', param_size + 1)
        end
      end
    end
  end
end
