module Luca
  class Ast
    class Lua
      # 逆クォート左辺値を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class BackQuoteLhs < Lhs
        @is_abstract = false

        # @return [Token]
        attr_reader :bquote_tk

        # @return [C::Expr] オペランドとなるCの式
        attr_reader :expr

        # @return [String, nil] Clangで解析する際のラベル名
        attr_accessor :label

        # @return [FFI::Clang::Type, nil] 解析結果の型
        attr_reader :type

        def initialize(bquote_tk, expr)
          expect_var[:bquote_tk].of_instance_to_be Token
          expect_var[:expr].of_instance_to_be C::Expr
          super(bquote_tk.line, expr.end_pos)
        end

        def print_by(cxt)
          cxt << bquote_tk << expr
        end

        def desugar_by(cxt)
          self
        end

        def set_parent_by(cxt)
          cxt.set_parent_of_ast(expr, self)
        end

        def collect_chunk_by(cxt)
          cxt.add_bquote(self)
        end

        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@expr)
          self
        end

        def gen_input_by(cxt)
          cxt << "#{label}:" << expr << ';' << cxt.endl
        end

        def type=(type)
          fail 'multiple setting @type' if instance_variable_defined?(:@type)

          expect_var[:type].to_be FFI::Clang::Type
          @type = type.canonical_rec
        end

        def analyze_stack_of_lvalue_by(cxt)
          0
        end

        def analyze_stack_of_assignment_by(cxt)
          0
        end

        def assignment_usage(cxt)
          0
        end

        def stack_usage(cxt)
          0
        end

        def gen_lvalue_code_by(cxt)
          expect_var[:cxt].to_be CodeGenerator

          # 左辺値を格納する変数(ポインタ)を宣言
          name = cxt.gen_name(:lhs)
          decl = type.gen_decl("*#{name}")
          cxt.newline
          cxt.write_lmacro(bquote_tk.line, cxt.logical_loc_of(bquote_tk.line).line)
          cxt << "#{decl} = &("
          cxt.in_lmacro_mode do
            cxt << expr
          end
          cxt << ');' << cxt.endl

          # 変数名を返す
          name
        end

        def gen_assignment_code_by(cxt, state, index)
          expect_var[:cxt].to_be CodeGenerator
          expect_var[:state].to_be Symbol
          expect_var[:index].to_be Integer

          name = state

          cxt.gen_checking_type_code(-1, type)
          if type.integer?
            cxt.add_stmt("*#{name} = #{cxt.build_api('tointeger', -1)}")
          elsif type.float_point?
            cxt.add_stmt("*#{name} = #{cxt.build_api('tonumber', -1)}")
          elsif type.string?
            cxt.add_stmt("*#{name} = #{cxt.build_api('tostring', -1)}")
          else
            if type.pointer?
              cxt << "if( #{cxt.build_api('type', -1)} == LUA_TUSERDATA ){" << cxt.endl
              cxt.next_level do
                cxt.add_stmt("*#{name} = *(#{type.gen_decl('*')})#{cxt.build_api('touserdata', -1)}")
              end
              cxt << '}' << cxt.endl
              cxt << 'else{' << cxt.endl
              cxt.next_level do
                cxt.add_stmt("*#{name} = NULL")
              end
              cxt << '}' << cxt.endl
            else
              cxt.add_stmt("*#{name} = *(#{type.gen_decl('*')})#{cxt.build_api('touserdata', -1)}")
            end
          end

          # 右辺値をpop
          cxt.call_api('pop', 1)
        end
      end
    end
  end
end
