module Luca
  class Ast
    class Lua
      # Luaの<tt>break</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class BreakStmt < Stmt
        @is_abstract = false

        # @return [Stmt] <tt>break</tt>するループ文
        attr_reader :outer_loop

        def initialize(pos)
          super(pos, pos)
        end

        def print_by(cxt)
          cxt << 'break'
        end

        def desugar_by(cxt)
          self
        end

        def set_parent_by(cxt); end

        def collect_chunk_by(cxt); end

        def analyze_jump_by(cxt)
          @outer_loop = cxt.current_loop

          fail "#{start_pos}: break is not inside loop" unless @outer_loop
        end

        def analyze_scope_by(cxt)
          self
        end

        def analyze_stack_by(cxt)
          0
        end

        def gen_input_by(cxt); end

        def gen_code_by(cxt)
          cxt.add_stmt('break')
        end
      end
    end
  end
end
