module Luca
  class Ast
    class Lua
      class CLocalDeclStmt < DeclStmt
        @is_abstract = false
        
        # @return [[C::DeclSpecifier]]
        attr_reader :decl_specifier_list
        
        # @return [C::Declarator]
        attr_reader :declarator
        
        # @return [Expr] 初期化式
        attr_reader :expr
        
        # @return [Program, FunctionLiteral] この<tt>clocal</tt>宣言を含むチャンク
        attr_reader :chunk
        
        # @return [String, nil]
        attr_accessor :label
        
        # @return [FFI::Clang::Type]
        attr_reader :type
        
        def initialize(start_pos, decl_specifier_list, declarator, expr)
          expect_var[:decl_specifier_list].of_instance_to_be ArrayOf(C::DeclSpecifier)
          expect_var[:declarator].of_instance_to_be C::Declarator
          expect_var[:expr].of_instance_to_be Expr
          super(start_pos, expr.end_pos)
        end
        
        # この文で宣言する変数名を返す
        #
        #
        # @return [Symbol]
        # 
        def name
          declarator.name || fail('no name given')
        end
        
        def print_by(cxt)
          cxt << 'clocal' << Printer.list(decl_specifier_list, ' ', true) << declarator << ' = ' << expr
        end
        
        def member_index=(index)
          fail 'multiple setting for @member_index' if instance_variable_defined?(:@member_index)
          expect_var[:index].to_be Integer
          @member_index = index
        end
        
        def member_index
          fail 'no setting for @member_index' unless instance_variable_defined?(:@member_index)
          @member_index
        end
        
        def type=(type)
          fail 'multiple setting @type' if instance_variable_defined?(:@type)
            
          expect_var[:type].to_be FFI::Clang::Type
          
          @type = type.canonical_rec
        end
        
        def desugar_by(cxt)
          @expr = cxt.desugar_ast(expr)
          self
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(decl_specifier_list, self)
          cxt.set_parent_of_ast(declarator, self)
          cxt.set_parent_of_ast(expr, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_ast(expr)
          cxt.current_chunk.add_c_local_decl(self)
          
          @chunk = cxt.current_chunk
        end
        
        def analyze_jump_by(cxt); end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@expr)
        end
        
        def gen_input_by(cxt)
          # 宣言を置いておく
          cxt << Printer.list(decl_specifier_list, ' ', true) << declarator << ';' << cxt.endl
          
          # 型を調べる為に変数を参照
          cxt << "#{label}: " << name << ';' << cxt.endl
          
          cxt << expr
        end
        
        def gen_env_member_by(cxt)
          expect_var[:cxt].to_be CodeGenerator
          cxt << cxt.list(decl_specifier_list, ' ', true)
          declarator.gen_env_member_by(cxt, member_index)
          cxt << ';' << cxt.endl
        end
        
        def analyze_stack_by(cxt)
          cxt.analyze_ast(expr, 1)
        end
        
        def gen_code_by(cxt)
          # 右辺値をスタックに積む
          cxt << expr
          
          c_env_pointer = cxt.c_env_pointer(chunk.id)
          cxt.gen_checking_type_code(-1, type)
          
          lhs = "#{c_env_pointer}->#{cxt.env_member_name(member_index)}"
          rhs = if type.integer?
                  cxt.build_api('tointeger', -1)
                elsif type.float_point?
                  cxt.build_api('tonumber', -1)
                elsif type.string?
                  cxt.build_api('tostring', -1)
                else
                  if type.pointer?
                    cond = cxt.build_api('type', -1) + ' == LUA_TNIL'
                    then_expr = 'NULL'
                    else_expr = "*(#{type.gen_decl('*')})#{cxt.build_api('touserdata', -1)}"
                    "#{cond} ? #{then_expr} : #{else_expr}"
                  else
                    "*(#{type.gen_decl('*')})#{cxt.build_api('touserdata', -1)}"
                  end
                end
          cxt.add_stmt("#{lhs} = #{rhs}")
          cxt.call_api('pop', 1)
        end
      end
    end
  end
end
