module Luca
  class Ast
    class Lua
      # Luaの関数呼出し文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class CallStmt < Stmt
        @is_abstract = false
        
        # @return [Expr] 関数呼出し式本体
        attr_reader :body
        
        def initialize(body)
          @body = expect_var[:body].to_be Expr
          super(body.start_pos, body.end_pos)
        end
        
        def print_by(cxt)
          cxt << body
        end
        
        def desugar_by(cxt)
          desugar_ivar_by(cxt, :@body)
          
          # 逆クォート関数呼び出しの場合，呼出し文であることを伝える．
          if body.is_a?(FunctionCallExpr) && body.func_expr.is_a?(BackQuoteExpr)
            BackQuoteCallStmt.new(body)
          else
            self
          end
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(body, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_ast(body)
        end
        
        def analyze_jump_by(cxt); end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@body)
        end
        
        def analyze_stack_by(cxt)
          cxt.analyze_ast(body, 0)
        end
        
        def gen_input_by(cxt)
          cxt.gen_input_of_ast(body)
        end
        
        def gen_code_by(cxt)
          cxt << body
        end
      end
    end
  end
end
