module Luca
  class Ast
    class Lua
      # Luaの<tt>do</tt>-<tt>end</tt>ブロックを表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class DoStmt < Stmt
        @is_abstract = false
        
        # @return [Block] 
        attr_reader :body
        
        def initialize(start_pos, end_pos, body)
          expect_var[:body].of_instance_to_be Block
          super(start_pos, end_pos)
        end
        
        def print_by(cxt)
          cxt << 'do' << Printer.endl
          cxt.next_level(body)
          cxt << Printer.endl << 'end'
        end
        
        def desugar_by(cxt)
          @body = cxt.desugar_ast(body)
          expect_var[:body].to_be Block
          self
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(body, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_ast(body)
        end
        
        def analyze_jump_by(cxt)
          cxt.analyze_ast(body)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@body)
        end
        
        def analyze_stack_by(cxt)
          cxt.analyze_ast(body)
        end
        
        def gen_input_by(cxt)
          cxt << body
        end
        
        def gen_code_by(cxt)
          cxt << body << cxt.endl
        end
      end
    end
  end
end
