module Luca
  class Ast
    class Lua
      # Luaの式を表すASTノード
      # 
      # @note このクラスは抽象クラスである
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class Expr < Lua
        @is_abstract = true
        
        def initialize(start_pos, end_pos)
          fail "#{self.class} is abstract class" if self.class == Expr
          
          super(start_pos, end_pos)
        end
        
        # 式がリテラルか否かを返す
        #
        #
        # @return [Boolean]
        # 
        def literal?
          self.is_a?(Literal)
        end
        
        # 式が定数リテラルか否かを返す
        #
        #
        # @return [Boolean] 
        # 
        def constant?
          self.is_a?(Constant)
        end
        
        def number?
          self.is_a?(NumberLiteral)
        end
        
        def string?
          self.is_a?(StringLiteral)
        end
        
        def truthy?
          !self.falsey?
        end
        
        def falsey?
          self.is_a?(False) || self.is_a?(Nil)
        end
        
        def compare_op_node?
          self.is_a? CompareOpNode
        end
        
        def not_op_node?
          self.is_a? NotOpNode
        end
        
        def multi?
          false
        end
      end
    end
  end
end
