module Luca
  class Ast
    class Lua
      # Luaの外部ローカル変数(上位値)の左辺値を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class ExternalLocalVarLhs < VarLhs
        @is_abstract = false

        # @return [[Integer, Integer]] アクセスパス({LocalVar#access_path}を参照)
        attr_reader :access_path

        def initialize(start_pos, name, access_path)
          expect_var[:access_path].of_instance_to_be TupleOf(Integer, Integer)
          super(pos, name)
        end

        def analyze_stack_of_lvalue_by(cxt)
          access_path.first + 1
        end

        def assignment_usage(cxt)
          0
        end

        def stack_usage(cxt)
          1
        end

        def gen_lvalue_code_by(cxt)
          expect_var[:cxt].to_be CodeGenerator

          # ローカル環境をひたすら辿る

          # 現在のローカル環境をpush
          cxt.call_api('pushvalue', cxt.env_index)

          depth = access_path.first
          i_var = cxt.gen_name('i')
          cxt << "for( int #{i_var} = 0; #{i_var} < #{depth}; #{i_var}++ ) {" << cxt.endl
          cxt.next_level do
            cxt.call_api('rawgeti', -1, cxt.prev_env_index)
          end
          cxt << '}' << cxt.endl

          # 不要な環境を削除
          cxt.call_api('copy', -1, -(depth + 1))
          cxt.call_api('pop', depth)
          nil
        end

        def gen_assignment_code_by(cxt, state, index)
          cxt.call_api('rawseti', index, cxt.lua_local_offset + access_path[1])
        end
      end
    end
  end
end
