module Luca
  class Ast
    class Lua
      # テーブルコンストラクタ({TableLiteral})中の個々のフィールドを表すASTノード
      # @note このクラスは抽象クラスである
      # @author autopp <autopp.inc@gmail.com>
      #
      class Field < Lua
        @is_abstract = true
        
        # @return [Expr] フィールドの値となる式
        attr_reader :value_expr
        alias_method :value, :value_expr
        
        def initialize(start_pos, value_expr)
          fail "#{self.class} is abstract class" if self.class == Field
          
          expect_var[:value_expr].of_instance_to_be Expr
          
          super(start_pos, value_expr.end_pos)
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(value_expr, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_ast(value_expr)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@value_expr)
        end
        
        def index?
          is_a?(IndexField)
        end
        
        def key_value?
          is_a?(KeyValueField)
        end
        
        def gen_input_by(cxt)
          cxt << value_expr
        end
      end
    end
  end
end
