module Luca
  class Ast
    class Lua
      # Luaの関数定義文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class FunctionDefStmt < Stmt
        @is_abstract = false
        @is_syntax_sugar = true
        
        # @return [[Symbol]] レシーバ及び関数名のリスト
        attr_reader :name_list
        
        # @return [[Symbol]] 引数名のリスト
        attr_reader :param_list
        
        # @return [Block] 本体
        attr_reader :body
        
        attr_reader :attr_names
        
        def initialize(start_pos, end_pos, name_list, param_list, varg, body)
          expect_var[:name_list].of_instance_to_be ArrayOf(Symbol) > 0
          expect_var[:param_list].of_instance_to_be ArrayOf(Symbol)
          expect_var[:varg].of_instance_to_be Boolean
          expect_var[:body].of_instance_to_be Block
          
          super(start_pos, end_pos)
        end
        
        # rubocop:disable Style/PredicateName
        # rubocop:disable Style/TrivialAccessors
        
        # 定義している関数が可変長引数を持っているか否かを返す
        #
        #
        # @return [Boolean] 
        # 
        def has_varg?
          @varg
        end
        # rubocop:enable Style/TrivialAccessors
        # rubocop:enable Style/PredicateName
        
        def print_by(cxt)
          cxt << 'function ' << Printer.list(name_list, '.') << '(' << Printer.list(param_list)
          if has_varg?
            cxt << ', ' unless param_list.empty?
            cxt << '...'
          end
          cxt << ')' << Printer.endl
          cxt.next_level(body)
          cxt << Printer.endl << 'end'
        end
        
        # 変数に対する関数リテラルの代入文へとdesugarする
        #
        # @param [<type>] cxt <description>
        #
        # @return [<type>] <description>
        # 
        def desugar_by(cxt)
          var = Var.new(start_pos, name_list.first)
          lhs = name_list[1...name_list.length].reduce(var) do |memo, name|
            AccessExpr.create_by_dot_access(memo, name, start_pos)
          end
          rhs = FunctionLiteral.new(start_pos, end_pos, param_list, has_varg?, body)
          cxt.desugar_ast(AssignStmt.new(start_pos, [lhs], [rhs]))
        end
      end
    end
  end
end
