module Luca
  class Ast
    class Lua
      # Luaの<tt>goto</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class GotoStmt < Stmt
        @is_abstract = false
        
        # @return [Symbol] ジャンプ先のラベル名
        attr_reader :name
        
        # @return [LabelStmt] ジャンプ先のラベル文
        attr_reader :dst
        
        def initialize(start_pos, end_pos, name)
          expect_var[:name].of_instance_to_be Symbol
          super(start_pos, end_pos)
        end
        
        def print_by(cxt)
          cxt << 'goto ' << name
        end
        
        def desugar_by(cxt)
          self
        end
        
        def set_parent_by(cxt)
        end

        def collect_chunk_by(cxt)
        end
        
        def analyze_scope_by(cxt)
          self
        end
        
        def analyze_jump_by(cxt)
          block = self.parent
          
          loop do
            @dst = block.label_list.find { |label| label.name == self.name }
            return if @dst # ラベルが見つかったら終了
            
            block = block.outer_block
            unless block.is_a?(block)
              fail "line #{self.start_pos}: no visible label `#{self.name}' for goto"
            end
          end
        end
        
        def gen_input_by(cxt)
        end
        
        def gen_code_by(cxt)
          fail NotImplementedError, 'goto stmt is not implemented'
        end
      end
    end
  end
end
