module Luca
  class Ast
    class Lua
      # Luaのテーブルコンストラクタ({TableLiteral})中の添字フィールドを表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class IndexField < Field
        @is_abstract = false
        
        def initialize(value_expr)
          super(value_expr.start_pos, value_expr)
        end
        
        def print_by(cxt)
          cxt << value_expr
        end
        
        def index=(i)
          fail 'multiple index setting' if @index
          
          expect_var[:i].to_be Integer
          @index = i
        end
        
        # このフィールドのインデックスを返す. 
        # 未設定の場合, RuntimeErrorを発生させる. 
        # 
        # @see index=
        #
        # @return [Integer] このフィールドのインデックス
        # @raise [RuntimeError] インデックスが未設定の場合
        # 
        def index
          fail 'no setting to index' unless @index
          
          @index
        end
        
        # desugarする. 
        # インデックスを与え, 値の部分をdesugarする
        #
        # @param [DesugarContext] cxt 
        #
        # @return [self] 
        # 
        def desugar_by(cxt)
          @index = cxt.next_field_index
          desugar_ivar_by(cxt, :@value_expr)
          expect_var[:@value_expr].to_be Expr
          self
        end
        
        def analyze_stack_by(cxt)
          cxt.let_safe_int(1) do
            @expected = cxt.current_expected_result
            cxt.analyze_ast(value_expr)
          end
        end
        
        def gen_code_by(cxt)
          if @expected == 1
            cxt << value_expr
            cxt.call_api('rawseti', -2, index)
          else
            cxt.let_safe_int do |n|
              cxt.add_stmt("#{n} = " + cxt.build_api('gettop'))
              cxt << value_expr
              i = cxt.gen_name(:i)
              decl = "int #{i} = " + cxt.build_api('gettop') + " - #{n}"
              cxt << "for( #{decl}; #{i} > 0; #{i}-- )"
              cxt.nest_block do
                cxt.call_api('rawseti', "-#{i}-1", "#{index} + #{i} - 1")
              end
            end
          end
        end
      end
    end
  end
end
