module Luca
  class Ast
    class Lua
      # 初期化式付きの<tt>protected</tt>宣言
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class InitializedProtectedDeclStmt < ProtectedDeclStmt
        # @return [[Expr]] 初期値リスト
        attr_reader :expr_list
        
        def initialize(kw_pos, var_list, expr_list)
          expect_var[:expr_list].of_instance_to_be ArrayOf(Expr) >= 1
          super(kw_pos, expr_list.last.end_pos, var_list)
        end
        
        def multi_rhs?
          expr_list.size > 0 && expr_last.multi?
        end
        
        def print_by(cxt)
          cxt << 'protected ' << Printer.list(var_list, ', ') << ' = ' << Printer.list(expr_list, ', ')
        end
        
        def desugar_by(cxt)
          desugar_ivar_by(cxt, :@expr_list)
          self
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(expr_list, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_list(expr_list)
        end
        
        def analyze_jump_by(cxt)
        end
        
        def gen_input_by(cxt)
          cxt.gen_input_of_list(expr_list)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@expr_list)
        end
        
        def analyze_stack_by(cxt)
          cxt.let_safe_int(1) do
            lhs_size, rhs_size = var_list.size, expr_list.size
            usage_list = if lhs_size >= rhs_size
                           # 最後以外は1つ積む前提で解析
                           usage_list = expr_list[0...-1].map.with_index do |rhs, i|
                             cxt.analyze_ast(rhs, 1) + i
                           end
                           
                           # 最後は左辺の個数に合わせて調整
                           n = lhs_size - rhs_size + 1
                           last_usage = cxt.analyze_ast(expr_list.last, n)
                           usage_list << last_usage + rhs_size - 1
                         else
                           # 左辺の個数までは1個ずつ積む前提で解析
                           forward_list = expr_list.take(lhs_size)
                           usage_list = forward_list.map.with_index do |rhs, i|
                             cxt.analyze_ast(rhs, i)
                           end
                           # 残りは全て捨てる前提で解析
                           backward_list = expr_list.drop(lhs_size)
                           usage_list + backward_list.map_with_index do |rhs, i|
                             cxt.analyze_ast(rhs, 0) + i + lhs_size
                           end
                         end
            usage_list.max
          end
        end
        
        def gen_code_by(cxt)
          # スタックの高さを確認
          cxt.let_safe_int do |height_var|
            if need_settop_call?
              cxt.add_stmt("#{height_var} = " + cxt.build_api('gettop'))
            end
            
            # 右辺値を全てスタックに積む
            expr_list.each do |rhs|
              cxt << rhs
            end
            
            # 左辺の数に合わせてスタックの高さを調整
            cxt.call_api('settop', "#{height_var} + #{var_list.size}") if need_settop_call?
            
            # 逆順に格納
            var_list.reverse_each do |var|
              cxt.call_api('pushstring', "\"#{var}\"")
              cxt.call_api('pushvalue', -2)
              cxt.call_api('rawset', 'LUA_REGISTRYINDEX')
              cxt.call_api('pop', 1)
            end
          end
        end
        
        # 代入処理前にsettopによるスタックの調整が必要かどうか
        #
        #
        # @return [Boolean]
        # 
        def need_settop_call?
          lhs_size, rhs_size = var_list.size, expr_list.size
          if lhs_size > rhs_size
            !multi_rhs?
          elsif lhs_size == rhs_size
            false
          else
            expr_list.drop(lhs_size).all?(&:multi?)
          end
        end
      end
    end
  end
end
