module Luca
  class Ast
    class Lua
      # テーブルコンストラクタ({TableLiteral})中の通常フィールドを表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class KeyValueField < Field
        @is_abstract = false
        
        # @return [Expr] フィールドのキーとなる式
        attr_reader :key_expr
        alias_method :key, :key_expr
        
        def initialize(start_pos, key_expr, value_expr)
          expect_var[:key_expr].of_instance_to_be Expr
          super(start_pos, value_expr)
        end
        
        def print_by(cxt)
          cxt << '[' << key_expr << '] = ' << value_expr
        end
        
        def desugar_by(cxt)
          self.desugar_ivar_by(cxt, :@key_expr, :@value_expr)
          expect_var[:@key_expr, :@value_expr].to_be Expr
          self
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(key_expr, self)
          super(cxt)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_ast(key_expr)
          super(cxt)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@key_expr)
          super(cxt)
        end
        
        def gen_input_by(cxt)
          cxt << key_expr
          super(cxt)
        end
        
        def analyze_stack_by(cxt)
          key_usage = cxt.analyze_ast(key_expr, 1)
          value_usage = cxt.analyze_ast(value_expr, 1)
          cxt.max(key_usage, value_usage + 1)
        end
        
        def gen_code_by(cxt)
          cxt << key_expr
          cxt << value_expr
          cxt.call_api('rawset', -3)
        end
      end
    end
  end
end
