module Luca
  class Ast
    class Lua
      # Luaのローカル関数定義文を表すASTノード
      # 
      # このASTノードはdesugaringフェーズによって, <tt>local</tt>宣言と関数リテラルの代入文へと変換される
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class LocalFunctionDefStmt < Stmt
        @is_abstract = false
        @is_syntax_sugar = true
        
        # @return [Symbol] 関数名
        attr_reader :name
        
        # @return [[Symbol]] 引数名のリスト
        attr_reader :param_list
        
        # @return [Block] 本体
        attr_reader :body
        
        def initialize(start_pos, end_pos, name, param_list, varg, body)
          expect_var[:name].of_instance_to_be Symbol
          expect_var[:param_list].of_instance_to_be ArrayOf(Symbol)
          expect_var[:varg].of_instance_to_be Boolean
          expect_var[:body].of_instance_to_be Block
          
          super(start_pos, end_pos)
        end
        
        # rubocop:disable Style/PredicateName
        # rubocop:disable Style/TrivialAccessors
        def has_varg?
          @varg
        end
        # rubocop:enable Style/TrivialAccessors
        # rubocop:enable Style/PredicateName
        
        
        def print_by(cxt)
          cxt << "local function #{name}(" << Printer.list(param_list)
          if has_varg?
            cxt << ', ' unless param_list.empty?
            cxt << '...'
          end
          cxt << ')' << Printer.endl
          cxt.next_level(body)
          cxt << Printer.endl << 'end'
        end
        
        def desugar_by(cxt)
          fail 'called dead method'
        end
      end
    end
  end
end
