module Luca
  class Ast
    class Lua
      # Luaのローカル変数を表すクラス
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class LocalVar < Var
        @is_abstract = false

        # @return [[Integer, Integer]] アクセスパス
        #   宣言されているスコープまでの距離と宣言されたインデックスのペア
        attr_reader :access_path

        attr_reader :id

        #
        #
        # @param [Integer] pos 行番号
        # @param [Symbol] name 変数名
        # @param [[Integer, Integer]] access_path アクセスパス
        #
        def initialize(pos, name, access_path, id)
          expect_var[:access_path].of_instance_to_be TupleOf(Integer, Integer)
          @id = id
          super(pos, name)
        end

        def external?
          access_path[0] > 0
        end

        def analyze_stack_by(cxt)
          access_path[0] + 1
        end

        def gen_code_by(cxt)
          if cxt.mode == CodeGenerator::MODE_CHUNK && !cxt.escape?(id)
            cxt.call_api('pushvalue', "#{cxt.stack_local_offset} + #{access_path[1]}")
          elsif !external?
            cxt.call_api('rawgeti', cxt.env_index, cxt.lua_local_offset + access_path[1])
          else
            cxt.call_api('rawgeti', cxt.env_index, cxt.prev_env_index)

            var_i = cxt.gen_name(:i)
            cxt << "for( int #{var_i} = #{access_path[0]}; #{var_i} > 1; #{var_i}-- )"
            cxt.nest_block do
              cxt.call_api('rawgeti', -1, cxt.prev_env_index)
            end
            cxt.call_api('rawgeti', -1, cxt.lua_local_offset + access_path[1])
            cxt.call_api('copy', -1, -access_path[0] - 1)
            cxt.call_api('pop', access_path[0])
          end
        end
      end
    end
  end
end
