module Luca
  class Ast
    class Lua
      # Luaのローカル変数の左辺値を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class LocalVarLhs < VarLhs
        @is_abstract = false

        # @return [Integer] ローカル環境内のインデックス
        attr_reader :var_index

        attr_reader :id

        def initialize(pos, name, var_index, id)
          expect_var[:var_index].of_instance_to_be Integer
          @id = id
          super(pos, name)
        end

        def analyze_stack_of_lvalue_by(cxt)
          cxt.escape?(id) ? 1 : 0
        end

        def assignment_usage(cxt)
          0
        end

        # ローカル変数を格納している環境1つのみを積む
        #
        # @return [1]
        #
        def stack_usage(cxt)
          cxt.escape?(id) ? 1 : 0
        end

        def gen_lvalue_code_by(cxt)
          expect_var[:cxt].to_be CodeGenerator

          if cxt.escape?(id)
            # 現在のローカル環境をpush
            cxt.call_api('pushvalue', cxt.env_index)
          end
          nil
        end

        def gen_assignment_code_by(cxt, state, index)
          if cxt.mode == CodeGenerator::MODE_CHUNK && !cxt.escape?(id)
            cxt.call_api('copy', -1, "#{cxt.stack_local_offset} + #{var_index}")
            cxt.call_api('pop', 1)
          else
            cxt.call_api('rawseti', index, cxt.lua_local_offset + var_index)
          end
        end
      end
    end
  end
end
