module Luca
  class Ast
    class Lua
      # Luaのメソッド定義文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class MethodDefStmt < Stmt
        @is_abstract = false
        @is_syntax_sugar = true
        
        # @return [[Symbol]] レシーバのリスト
        attr_reader :recv_list
        
        # @return [Symbol] メソッド名
        attr_reader :name
        
        # @return [[Symbol]] 引数名のリスト
        attr_reader :param_list
        
        # @return [Block] 本体
        attr_reader :body
        
        def initialize(start_pos, end_pos, recv_list, name, param_list, varg, body)
          expect_var[:recv_list].of_instance_to_be ArrayOf(Symbol) > 1
          expect_var[:name].of_instance_to_be Symbol
          expect_var[:param_list].of_instance_to_be ArrayOf(Symbol)
          expect_var[:varg].of_instance_to_be Boolean
          expect_var[:body].of_instance_to_be Block
          super(start_pos, end_pos)
        end
        
        # rubocop:disable Style/PredicateName
        # rubocop:disable Style/TrivialAccessors
        
        # 定義しているメソッドが可変長引数を持つか否かを返す
        #
        #
        # @return [Boolean] 
        # 
        def has_varg?
          @varg
        end
        # rubocop:enable Style/TrivialAccessors
        # rubocop:enable Style/PredicateName
        
        def print_by(cxt)
          cxt << 'function ' << Printer.list(recv_list, '.') << ":#{name}" << '(' << Printer.list(param_list)
          if has_varg
            cxt << ', ' unless param_list.empty
            cxt << '...'
          end
          cxt << ')' << Printer.endl
          cxt.next_level(body)
          cxt << Printer.endl << 'end'
        end
        
        # テーブルへの代入文へとdesugarする
        #
        # @param [<type>] cxt <description>
        #
        # @return [<type>] <description>
        # 
        def desugar_by(cxt)
          var = Var.new(start_pos, recv_list.first)
          var = recv_list[1...recv_list.length].reduce(var) do |memo, item|
            AccessExpr.create_by_dot_access(start_pos, memo, item)
          end
          lhs = AccessExpr.create_by_dot_access(start_pos, var, name)
          
          # 本体をdesugar
          body = cxt.desugar_ast(body)
          rhs = FunctionLiteral.new(start_pos, end_pos, param_list.unshift(:self), has_varg?, body)
          
          AssignStmt.new(start_pos, [lhs], [rhs])
        end
      end
    end
  end
end
