module Luca
  class Ast
    class Lua
      # Luaの括弧付き式を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class ParenthesesExpr < Expr
        @is_abstract = false
        
        # @return [Expr] 括弧内の式
        attr_reader :inner_expr
        
        alias_method :expr, :inner_expr
        
        def initialize(start_pos, end_pos, inner_expr)
          expect_var[:inner_expr].of_instance_to_be Expr
          super(start_pos, end_pos)
        end
        
        def print_by(cxt)
          cxt << '(' << inner_expr << ')'
        end
        
        # desugar後の括弧内の式が定数リテラルだった場合, 括弧を外す
        # 
        # これによって定数畳み込みが行える
        #
        # @param [DesugarContext] cxt 
        #
        # @return [Expr] 
        # 
        def desugar_by(cxt)
          desugar_ivar_by(cxt, :@inner_expr)
          
          if inner_expr.constant?
            inner_expr
          elsif !inner_expr.multi?
            inner_expr
          else
            self
          end
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(inner_expr, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_ast(inner_expr)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@inner_expr)
          self
        end
        
        def analyze_stack_by(cxt)
          cxt.analyze_ast(inner_expr, 1)
        end
        
        def gen_input_by(cxt)
          cxt.gen_input_of_ast(inner_expr)
        end
        
        def gen_code_by(cxt)
          cxt << inner_expr
        end
      end
    end
  end
end
