module Luca
  class Ast
    class Lua
      # Luaの<tt>repeat</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class RepeatStmt < Stmt
        @is_abstract = false
        @is_syntax_sugar = true
        
        # @return [Block] 本体
        attr_reader :body
        
        # @return [Expr] 条件式
        attr_reader :cond_expr
        
        def initialize(start_pos, body, cond_expr)
          expect_var[:body].of_instance_to_be Block
          expect_var[:cond_expr].of_instance_to_be Expr
          super(start_pos, cond_expr.end_pos)
        end
        
        def print_by(cxt)
          cxt << 'repeat' << Printer.endl
          cxt.next_level(body)
          cxt << Printer.endl << 'until ' << cond_expr
        end
        
        # desugarする
        # 
        # 本体が<tt>return</tt>文を持たない場合, 最後に<tt>if</tt>, <tt>break</tt>を持つ<tt>while</tt>文に変換する
        # 持つ場合は単なる<tt>do</tt>-<tt>end</tt>文に変換する
        #
        # @param [DesugarContext] cxt 
        #
        # @return [Ast] 
        # 
        def desugar_by(cxt)
          if body.return_stmt
            cxt.desugar_ast(DoStmt.new(start_pos, end_pos, body))
          else
            break_stmt = BreakStmt.new(end_pos)
            if_stmt = IfStmt.new(end_pos, end_pos, cond_expr, Block.new([break_stmt]), [])
            new_block = Block.new([*body.stmt_list, if_stmt])
            
            cxt.desugar_ast(WhileStmt.new(start_pos, end_pos, True.new(start_pos), new_block))
          end
        end
      end
    end
  end
end
