module Luca
  class Ast
    class Lua
      # Luaのreturn文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class ReturnStmt < Lua
        @is_abstract = false

        # @return [[Expr]] 戻り値の式リスト
        attr_reader :expr_list

        def initialize(start_pos, expr_list = [], semicolon: false)
          @expr_list = expect_var[:expr_list].to_be ArrayOf(Expr)
          if semicolon
            expect_var[:semicolon].to_be Integer
            super(start_pos, semicolon)
          else
            if expr_list.empty?
              super(start_pos, start_pos)
            else
              super(start_pos, expr_list.last.end_pos)
            end
          end
        end

        def multi_return?
          !expr_list.empty? && expr_list.last.multi?
        end

        def print_by(cxt)
          cxt << 'return ' << Printer.list(expr_list, ', ')
        end

        def desugar_by(cxt)
          @expr_list = cxt.desugar_list(expr_list)
          self
        end

        def set_parent_by(cxt)
          cxt.set_parent_of_list(expr_list, self)
        end

        def collect_chunk_by(cxt)
          cxt.collect_from_list(expr_list)
        end

        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@expr_list)
        end

        def gen_input_by(cxt)
          cxt.gen_input_of_list(expr_list)
        end

        def analyze_stack_by(cxt)
          cxt.let_safe_int(1) do
            usage_list = expr_list[0...-1].map.with_index do |expr, i|
              cxt.analyze_ast(expr, 1) + i
            end
            usage = usage_list.max || 0

            last_usage = if expr_list.empty?
                           0
                         else
                           cxt.analyze_ast(expr_list.last, cxt.multi_ret) + expr_list.size - 1
                         end

            cxt.max(usage, last_usage)
          end
        end

        def gen_code_by(cxt)
          cxt.let_safe_int do |h_var|
            if multi_return?
              # スタックの高さを記憶
              cxt.add_stmt("#{h_var} = #{cxt.build_api('gettop')}")
            end
            # 戻り値を積む
            expr_list.each do |arg|
              cxt << arg
            end

            if cxt.mode == CodeGenerator::MODE_FUNCTION
              if multi_return?
                size = cxt.build_api('gettop') + " - #{h_var}"
                cxt.add_stmt("return #{size}")
              else
                cxt.add_stmt("return #{expr_list.size}")
              end
            else
              cxt.add_stmt("goto #{cxt.end_of_lua_stmt}")
            end
          end
        end
      end
    end
  end
end
