module Luca
  class Ast
    class Lua
      # <tt>_setmetatable</tt>式を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class SetmetatableExpr < Expr
        # @return [Expr] メタテーブルの設定対象の式
        attr_reader :object_expr
        alias_method :object, :object_expr
        
        # @return [expr] 設定するメタテーブルを表す式
        attr_reader :metatable_expr
        alias_method :metatable, :metatable_expr
        
        def initialize(start_pos, end_pos, object_expr, metatable_expr)
          expect_var[:object_expr].of_instance_to_be Expr
          expect_var[:metatable_expr].of_instance_to_be Expr
          super(start_pos, end_pos)
        end
        
        def print_by(cxt)
          cxt << '_setmetatable' << '(' << object_expr << ', ' << metatable_expr << ')'
        end
        
        def desugar_by(cxt)
          desugar_ivar_by(cxt, :@object_expr, :@metatable_expr)
          self
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_ast(object_expr, self)
          cxt.set_parent_of_ast(metatable_expr, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_ast(object_expr)
          cxt.collect_from_ast(metatable_expr)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@object_expr, :@metatable_expr)
        end
        
        def gen_input_by(cxt)
          cxt.gen_input_of_ast(object_expr)
          cxt.gen_input_of_ast(metatable_expr)
        end
        
        def analyze_stack_by(cxt)
          cxt.expect_result(1) do
            [cxt.analyze_ast(object_expr), cxt.analyze_ast(metatable_expr) + 1].max
          end
        end
        
        def gen_code_by(cxt)
          cxt << object_expr << cxt.endl << metatable_expr
          cxt.call_api('setmetatable', -2)
        end
      end
    end
  end
end
