module Luca
  class Ast
    class Lua
      # Luaのテーブルコンストラクタを表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class TableLiteral < Literal
        @is_abstract = false
        
        # @return [[Field]] フィールドのリスト
        attr_reader :field_list
        
        def initialize(start_pos, end_pos, field_list)
          expect_var[:field_list].of_instance_to_be ArrayOf(Field)
          super(start_pos, end_pos)
        end
        
        def multi_last_field?
          !field_list.empty? && field_list.last.multi?
        end
        
        ONE_LINE_THRESHOLD = 5
        def print_by(cxt)
          if field_list.length <= ONE_LINE_THRESHOLD
            cxt << '{ ' << Printer.list(field_list, ', ') << ' }'
          else
            cxt << '{' << Printer.endl
            cxt.next_level do
              field_list.each do |field|
                cxt << field << ',' << Printer.endl
              end
            end
            cxt << '}'
          end
        end
        
        def desugar_by(cxt)
          cxt.let_field_index do
            self.desugar_ivar_by(cxt, :@field_list)
          end
          self
        end
        
        def set_parent_by(cxt)
          cxt.set_parent_of_list(field_list, self)
        end
        
        def collect_chunk_by(cxt)
          cxt.collect_from_list(field_list)
        end
        
        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@field_list)
        end
        
        def gen_input_by(cxt)
          cxt.gen_input_of_list(field_list)
        end
        
        def analyze_stack_by(cxt)
          usage_list = field_list[0...-1].map do |field|
            cxt.analyze_ast(field, 1)
          end
          usage = usage_list.max || 0
          
          last_usage = if field_list.empty?
                         0
                       else
                         cxt.analyze_ast(field_list.last, cxt.multi_ret)
                       end
          cxt.max(usage, last_usage + 1)
        end
        
        def gen_code_by(cxt)
          narr, nrec = field_list.partition(&:index?).map(&:size)
          
          cxt.call_api('createtable', narr, nrec)
          field_list.each do |field|
            cxt << field
          end
        end
      end
    end
  end
end
