module Luca
  class Ast
    class Lua
      # Luaの変数を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class Var < Literal
        LOCAL = :local
        PROTECTED = :protected
        GLOBAL = :global

        @is_abstract = false

        # @return [Symbol] 変数名
        attr_reader :name

        alias_method :pos, :start_pos

        def initialize(pos, name)
          expect_var[:name].of_instance_to_be Symbol
          super(pos, pos)
        end

        def to_lhs
          VarLhs.new(pos, name)
        end

        def to_local(access_path, id)
          LocalVar.new(pos, name, access_path, id)
        end

        def to_global
          GlobalVar.new(pos, name)
        end

        def to_protected
          ProtectedVar.new(pos, name)
        end

        def print_by(cxt)
          cxt << name
        end

        def desugar_by(cxt)
          self
        end

        def set_parent_by(cxt)
        end

        def collect_chunk_by(cxt)
        end

        def analyze_scope_by(cxt)
          kind, access_path, id = cxt.lookup_lua_var(name)

          case kind
          when LOCAL
            to_local(access_path, id)
          when GLOBAL
            to_global
          when PROTECTED
            to_protected
          else
            fail "undefined kind #{kind.inspect}"
          end
        end

        def gen_input_by(cxt); end
      end
    end
  end
end
