module Luca
  class Ast
    class Lua
      # Luaの変数の左辺値を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class VarLhs < Lhs
        @is_abstract = false

        # @return [Symbol] 変数名
        attr_reader :name

        alias_method :pos, :start_pos

        def initialize(pos, name)
          expect_var[:name].of_instance_to_be Symbol
          super(pos, pos)
        end

        def to_local(access_path, id)
          if access_path.first == 0
            LocalVarLhs.new(pos, name, access_path[1], id)
          else
            ExternalLocalVarLhs.new(pos, name, access_path)
          end
        end

        def to_global
          GlobalVarLhs.new(pos, name)
        end

        def to_protected
          ProtectedVarLhs.new(pos, name)
        end

        def print_by(cxt)
          cxt << name
        end

        def desugar_by(cxt)
          self
        end

        def set_parent_by(cxt)
        end

        def collect_chunk_by(cxt)
        end

        def analyze_scope_by(cxt)
          kind, access_path, id = cxt.lookup_lua_var(name)
          case kind
          when Var::LOCAL
            to_local(access_path, id)
          when Var::GLOBAL
            to_global
          when Var::PROTECTED
            to_protected
          else
            fail "undefined kind #{kind.inspect}"
          end
        end

        def gen_input_by(cxt)
        end
      end
    end
  end
end
