module Luca
  class Ast
    class Lua
      # Luaの可変長引数式を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class Vararg < Expr
        @is_abstract = false
        
        def initialize(pos)
          super(pos, pos)
        end
        
        def multi?
          true
        end
        
        def print_by(cxt)
          cxt << '...'
        end
        
        def desugar_by(cxt)
          self
        end
        
        def set_parent_by(cxt)
        end
        
        def collect_chunk_by(cxt)
        end
        
        def analyze_scope_by(cxt)
          fail 'cannot use `...\' outside a varg function' if cxt.current_function.varg?
          self
        end
        
        def gen_input_by(cxt); end
        
        def analyze_stack_by(cxt)
          @expected = cxt.current_expected_result
          @expected == cxt.multi_ret ? 0 : @expected
        end
        
        def gen_code_by(cxt)
          i = cxt.gen_name(:i)
          if @expected == Analyzer::StackAnalyzer::MULTI_RET
            cxt << "for( int #{i} = 0; #{i} < #{cxt.vararg_size}; #{i}++ )"
            cxt.nest_block do
              cxt.call_api('pushvalue', "#{cxt.vararg_offset} + #{i}")
            end
          else
            cxt.add_stmt("int #{i}")
            cxt << "for( #{i} = 0; #{i} < #{cxt.vararg_size} && #{i} < #{@expected}; #{i}++)"
            cxt.nest_block do
              cxt.call_api('pushvalue', "#{cxt.vararg_offset} + #{i}")
            end
            cxt << "for( ; #{i} < #{@expected}; #{i}++ )"
            cxt.nest_block do
              cxt.call_api('pushnil')
            end
          end
        end
      end
    end
  end
end
