module Luca
  class Ast
    class Lua
      # Luaの<tt>while</tt>文を表すASTノード
      #
      # @author autopp <autopp.inc@gmail.com>
      #
      class WhileStmt < Stmt
        @is_abstract = false

        # @return [Expr] 条件式
        attr_reader :cond_expr

        # @return [Block] 本体
        attr_reader :body

        def initialize(start_pos, end_pos, cond_expr, body)
          expect_var[:cond_expr].of_instance_to_be Expr
          expect_var[:body].of_instance_to_be Block
          super(start_pos, end_pos)
        end

        def print_by(cxt)
          cxt << 'while ' << cond_expr << ' do' << Printer.endl
          cxt.next_level(body)
          cxt << Printer.endl << 'end'
        end

        def desugar_by(cxt)
          desugar_ivar_by(cxt, :@cond_expr, :@body)
          self
        end

        def set_parent_by(cxt)
          cxt.set_parent_of_ast(cond_expr, self)
          cxt.set_parent_of_ast(body, self)
        end

        def collect_chunk_by(cxt)
          cxt.collect_from_ast(cond_expr)
          cxt.collect_from_ast(body)
        end

        def analyze_jump_by(cxt)
          cxt.nest_loop(self) do
            cxt.analyze_ast(body)
          end
        end

        def analyze_scope_by(cxt)
          analyze_scope_of_ivar_by(cxt, :@cond_expr, :@body)
        end

        def analyze_stack_by(cxt)
          cond_usage = cxt.analyze_ast(cond_expr, 1)
          body_usage = cxt.analyze_ast(body, 0)
          cxt.max(cond_usage, body_usage)
        end

        def gen_input_by(cxt)
          cxt << cond_expr << body
        end

        def gen_code_by(cxt)
          cxt << 'while( 1 )'
          cxt.nest_block do
            if cond_expr.constant?
              cxt.add_stmt('break') if cond_expr.falsey?
            elsif cond_expr.compare_op_node?
              cxt << cond_expr.left << cxt.endl
              cxt << cond_expr.right << cxt.endl
              cmp_expr = cxt.build_api('compare', -2, -1, cond_expr.operator.api_const)
              gen_break_code_by(cxt, cmp_expr, 2)
            elsif cond_expr.not_op_node?
              cxt << cond_expr
              cmp_expr = '!' + cxt.build_api('toboolean', -1)
              gen_break_code_by(cxt, cmp_expr, 1)
            else
              cxt << cond_expr
              cmp_expr = cxt.build_api('toboolean', -1)
              gen_break_code_by(cxt, cmp_expr, 1)
            end

            cxt << body
          end
        end

        def gen_break_code_by(cxt, cmp_expr, pop_count)
          cxt << "if( !#{cmp_expr} )"
          cxt.nest_block do
            cxt.call_api('pop', pop_count)
            cxt.add_stmt('break')
          end
          cxt.call_api('pop', pop_count)
        end
      end
    end
  end
end
