module Luca
  # ASTをdesugarするためのクラス
  #
  # @author autopp <autopp.inc@gmail.com>
  #
  class DesugarContext
    include Varspec::BuiltinMatcher
    
    def initialize
      @index_count_stack = []
    end
    
    def desugar_program(program)
      expect_var[:program].to_be Ast::C::Program
      ret = program.translate_lua_by(self, :desugar_ast)
      expect_var[:ret].to_be Ast::C::Program
      
      ret
    end
    
    # ASTノードをdesugarする
    # 
    # 各ASTクラスは<tt>desugar_by</tt>メソッドを実装する必要がある
    #
    # @param [Ast] ast desugarするAST
    #
    # @return [Ast] desugar済みのAST
    # 
    def desugar_ast(ast)
      expect_var[:ast].to_be Ast
      ret = ast.enter_desugaring_by(self)
      expect_var[:ret].to_be Ast
      
      ret
    end
    
    # リストの各要素のASTをdesugarしたリストを返す
    #
    # @param [[Ast]] ast_list desugarするASTのリスト
    #
    # @return [[Ast]] desugar済みのASTのリスト
    # 
    def desugar_list(ast_list)
      expect_var[:ast_list].to_be ArrayOf(Ast)
      ast_list.map do |ast|
        self.desugar_ast(ast)
      end
    end
    
    # テーブルコンストラクタ用のカウンタスタックを積み, 与えられたブロックを評価する. 
    # 
    #
    # @return [object] ブロックの実行結果
    def let_field_index
      @index_count_stack.push << 0
      ret = yield
      @index_count_stack.pop
      ret
    end
    
    # 現在のコンテキストで, 次のテーブルコンストラクタのインデックスを返す. 
    #  
    #  
    # @return [Integer] 次のインデックス
    # 
    def next_field_index
      @index_count_stack[-1] += 1
    end
  end
end
