module Luca
  # プリプロセス後のlineマクロの情報を持つクラス．
  #
  # @author autopp <autopp.inc@gmail.com>
  #
  class LocationMap
    include Varspec::BuiltinMatcher
    
    # @return [String]
    attr_reader :filename
        
    def initialize(filename, loc_list)
      expect_var[:filename].of_instance_to_be String
      expect_var[:loc_list].to_be ArrayOf(TupleOf(Integer, Integer, String, String))
      
      @lmacro_list = [LineMacro.new(0, 1, "\"#{filename}\""), *loc_list.map(&LineMacro.method(:new_by_ary))]
      
      lmacro_list.drop(1).each.with_index(1) do |x, i|
        fail 'not sorted' if x.actual_line < lmacro_list[i - 1].actual_line
      end
    end
    
    def logical_loc_of(actual_line)
      prev = @lmacro_list.first
      lmacro_list.drop(1).each do |lmacro|
        if actual_line <= lmacro.actual_line
          break
        else
          prev = lmacro
        end
      end
      Location.new(prev.logical_line_of(actual_line), prev.filename)
    end
    
    def lmacro_list
      @lmacro_list.dup
    end
    
    def logical_filename_of(actual_line)
      logical_loc(actual_line).filename
    end
    
    def logical_line_of(actual_line)
      logical_loc(actual_line).line
    end
    
    def find_lmacro(line)
      lmacro_list.find do |lmacro|
        lmacro.actual_line == line
      end
    end
    
    class Location
      # @return [Integer]
      attr_reader :line
      # @return [String]
      attr_reader :filename
      
      def initialize(line, filename)
        expect_var[:line].of_instance_to_be Integer
        expect_var[:filename].of_instance_to_be String
      end
      
      def to_s
        "#{filename}:#{line}"
      end
    end
    
    class LineMacro
      # @return [Integer] 実際の行番号
      attr_reader :actual_line
      
      # @return [Integer] 行番号
      attr_reader :next_line
      
      # @return [String] ファイル名
      attr_reader :filename
      
      # @return [String] その他
      attr_reader :misc
      
      def initialize(actual_line, next_line, filename, misc = '')
        @actual_line = actual_line
        @next_line = next_line
        @filename = filename.dup.freeze
        @misc = misc.dup.freeze
      end
      
      def logical_line_of(line)
        fail "line(#{line}) is not greater than #{actual_line}" unless actual_line < line
        next_line + (line - actual_line - 1)
      end
      
      def to_s
        "# #{next_line} \"#{filename}\"#{misc}"
      end
      
      def self.new_by_ary(args)
        new(*args)
      end
    end
  end
end
